# -*- coding: utf-8 -*-
__version__ = "0.7.1"

from .amplpypi import (
    path,
    find,
    run_command as run,
    activate_license as activate,
    install_modules as install,
    uninstall_modules as uninstall,
    load_modules as load,
    unload_modules as unload,
    preload_modules as preload,
    installed_modules as installed,
    available_modules as available,
    generate_requirements as requirements,
)

from .utils import cloud_platform_name

from .commands import _main



# -*- coding: utf-8 -*-
__version__ = "0.7.1"
from .notebooks import (
    ampl_notebook,
)

from .modules import (
    activate as activate_license,
    cloud_platform_name,
)

from .utils import (
    register_magics,
    add_to_path,
)

_SUPPORT_MESSAGE = """

Please report any bugs at: https://github.com/ampl/amplpy

For support/feedback go to https://discuss.ampl.com or e-mail <support@ampl.com>
"""

try:
    from IPython import get_ipython

    register_magics(ampl_object="ampl", globals_=get_ipython().user_global_ns)
except:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from . import TestBase
import unittest
import amplpy


class TestProperties(TestBase.TestBase):
    """Test Properties."""

    def testVarProp(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.var["x"] = 10
        self.assertEqual(ampl.var["x"].value(), ampl.getVariable("x").value())
        self.assertEqual(len(list(ampl.var)), len(list(ampl.getVariables())))

    def testConProp(self):
        ampl = self.ampl
        ampl.eval("var x; s.t. c: x = 10;")
        ampl.con["c"] = 10
        self.assertEqual(ampl.con["c"].dual(), ampl.getConstraint("c").dual())
        self.assertEqual(len(list(ampl.con)), len(list(ampl.getConstraints())))

    def testObjProp(self):
        ampl = self.ampl
        ampl.eval("var x; maximize obj: x;")
        self.assertEqual(ampl.obj["obj"].name(), ampl.getObjective("obj").name())
        self.assertEqual(len(list(ampl.obj)), len(list(ampl.getObjectives())))

    def testSetProp(self):
        ampl = self.ampl
        ampl.eval("set s;")
        ampl.set["s"] = [1, 2, 3]
        self.assertEqual(ampl.set["s"].name(), ampl.getSet("s").name())
        self.assertEqual(ampl.set["s"].size(), 3)
        self.assertEqual(len(list(ampl.set)), len(list(ampl.getSets())))

    def testParamProp(self):
        ampl = self.ampl
        ampl.eval("param p1; param p2{1..2};")
        self.assertEqual(ampl.param["p1"].name(), ampl.getParameter("p1").name())
        ampl.param["p1"] = 3
        self.assertEqual(ampl.param["p1"].value(), 3)
        ampl.param["p2"] = {1: 10, 2: 20}
        self.assertEqual(ampl.param["p2"][1], 10)
        self.assertEqual(ampl.param["p2"][2], 20)
        self.assertEqual(len(list(ampl.param)), len(list(ampl.getParameters())))

    def testOptionProp(self):
        ampl = self.ampl
        ampl.option["solver"] = "gurobi"
        self.assertEqual(ampl.option["solver"], ampl.getOption("solver"))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import sys
import os

_parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
_vendor_dir = os.path.join(_parent_dir, "vendor")
if _vendor_dir not in sys.path:
    sys.path.append(_vendor_dir)

try:
    from ampltools import *
    from ampltools import __version__
except ImportError:
    print(
        "Failed to import 'ampltools'. Install or upgrade with:\n"
        "   $ python -m pip install ampltools --upgrade\n"
    )
    raise

try:
    from ampltools import _SUPPORT_MESSAGE
except Exception:
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from .. import modules
from . import TestBase


class TestInstall(TestBase.TestBase):
    """Test ampltools.modules: install, uninstall, available, installed, and requirements."""

    def test_install_verbose(self):
        self._test_install(verbose=True)

    def test_install_silent(self):
        self._test_install(verbose=False)

    def _test_install(self, verbose):
        self.assertEqual(modules.installed(), [])

        modules.install("highs", verbose=verbose)
        self.assertEqual(modules.installed(), ["base", "highs"])

        with self.assertRaises(Exception) as context:
            modules.install("high", verbose=verbose)
        self.assertTrue(
            "AMPL module 'high' is not available." in str(context.exception)
        )

        with self.assertRaises(Exception) as context:
            modules.install("ipopt", verbose=verbose)
        self.assertTrue(
            "AMPL module 'ipopt' is not available. It is included in module 'coin'."
            in str(context.exception)
        )

        modules.install(["coin"], verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base", "highs", "coin"})

        modules.uninstall("highs", verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base", "coin"})

        with self.assertRaises(Exception) as context:
            modules.uninstall("base")

        self.assertTrue(
            "Base module cannot be uninstalled alone. You need to uninstall all modules."
            in str(context.exception)
        )

        modules.uninstall("coin", verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base"})

        modules.uninstall("base", verbose=verbose)
        self.assertEqual(modules.installed(), [])

        modules.install(["cbc", "highs"], verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base", "highs", "cbc"})

        modules.uninstall(verbose=verbose)
        self.assertEqual(modules.installed(), [])

    def test_available(self):
        self.assertTrue(isinstance(modules.available(), list))
        self.assertGreaterEqual(len(modules.available()), 5)
        self.assertTrue("base" in modules.available())
        self.assertTrue("highs" in modules.available())
        self.assertTrue("coin" in modules.available())
        self.assertTrue("cbc" in modules.available())
        self.assertFalse("ipopt" in modules.available())

    def test_requirements(self):
        self.assertEqual(modules.installed(), [])
        ampl_pypi = "--index-url https://pypi.ampl.com"
        extra_pypi = "--extra-index-url https://pypi.org/simple"

        requirements = modules.requirements().strip().split("\n")
        print("requirements:", requirements)
        self.assertEqual(requirements[0], ampl_pypi)
        self.assertEqual(requirements[1], extra_pypi)
        self.assertEqual(len(requirements), 2)

        modules.install(["cbc", "highs"])

        requirements = modules.requirements().strip().split("\n")
        self.assertEqual(requirements[0], ampl_pypi)
        self.assertEqual(requirements[1], extra_pypi)
        self.assertTrue(
            any(req.startswith("ampl_module_base==") for req in requirements[2:])
        )
        self.assertTrue(
            any(req.startswith("ampl_module_cbc==") for req in requirements[2:])
        )
        self.assertTrue(
            any(req.startswith("ampl_module_highs==") for req in requirements[2:])
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from . import TestBase
import unittest
import amplpy


class TestIterators(TestBase.TestBase):
    """Test Iterators."""

    def testSetIterators(self):
        ampl = self.ampl
        ampl.eval("set x;")
        self.assertEqual(len(ampl.getSets()), ampl.getSets().size())
        self.assertEqual(ampl.getSets().size(), len(list(ampl.getSets())))
        self.assertEqual(ampl.getSets().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.getSets().size(), 0)
        ampl.eval("set x;")
        self.assertEqual(ampl.getSets().size(), 1)
        ampl.eval("set y{1..2};")
        self.assertEqual(ampl.getSets().size(), 2)
        ampl.eval('set z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getSets().size(), 3)
        ampl.eval("set xx{1..2,4..5};")
        self.assertEqual(ampl.getSets().size(), 4)
        ampl.eval("set yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.getSets().size(), 5)
        ampl.eval('set zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getSets().size(), 6)
        # self.assertEqual(ampl.getSet("x").numInstances(), len(dict(ampl.getSet("x"))))
        self.assertEqual(ampl.getSet("x").numInstances(), 1)
        self.assertEqual(ampl.getSet("y").numInstances(), 2)
        self.assertEqual(ampl.getSet("z").numInstances(), 4)
        self.assertEqual(ampl.getSet("xx").numInstances(), 4)
        self.assertEqual(ampl.getSet("yy").numInstances(), 8)
        self.assertEqual(ampl.getSet("zz").numInstances(), 8)
        self.assertEqual(max(s.numInstances() for name, s in ampl.getSets()), 8)
        self.assertEqual(ampl.getSet("x").name(), ampl.getSets()["x"].name())

    def testParameterIterators(self):
        ampl = self.ampl
        ampl.eval("param x;")
        self.assertEqual(len(ampl.getParameters()), ampl.getParameters().size())
        self.assertEqual(ampl.getParameters().size(), len(list(ampl.getParameters())))
        self.assertEqual(ampl.getParameters().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.getParameters().size(), 0)
        ampl.eval("param x;")
        self.assertEqual(ampl.getParameters().size(), 1)
        ampl.eval("param y{1..2};")
        self.assertEqual(ampl.getParameters().size(), 2)
        ampl.eval('param z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getParameters().size(), 3)
        ampl.eval("param xx{1..2,4..5};")
        self.assertEqual(ampl.getParameters().size(), 4)
        ampl.eval("param yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.getParameters().size(), 5)
        ampl.eval('param zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getParameters().size(), 6)
        self.assertEqual(
            ampl.getParameter("x").numInstances(), len(dict(ampl.getParameter("x")))
        )
        self.assertEqual(ampl.getParameter("x").numInstances(), 1)
        self.assertEqual(ampl.getParameter("y").numInstances(), 2)
        self.assertEqual(ampl.getParameter("z").numInstances(), 4)
        self.assertEqual(ampl.getParameter("xx").numInstances(), 4)
        self.assertEqual(ampl.getParameter("yy").numInstances(), 8)
        self.assertEqual(ampl.getParameter("zz").numInstances(), 8)
        self.assertEqual(max(p.numInstances() for name, p in ampl.getParameters()), 8)
        self.assertEqual(
            ampl.getParameter("x").name(), ampl.getParameters()["x"].name()
        )

    def testVariableIterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        self.assertEqual(len(ampl.getVariables()), ampl.getVariables().size())
        self.assertEqual(ampl.getVariables().size(), len(list(ampl.getVariables())))
        self.assertEqual(ampl.getVariables().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.getVariables().size(), 0)
        ampl.eval("var x;")
        self.assertEqual(ampl.getVariables().size(), 1)
        ampl.eval("var y{1..2};")
        self.assertEqual(ampl.getVariables().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.getVariables().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        self.assertEqual(ampl.getVariables().size(), 4)
        ampl.eval("var yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.getVariables().size(), 5)
        ampl.eval('var zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.getVariables().size(), 6)
        self.assertEqual(
            ampl.getVariable("x").numInstances(), len(dict(ampl.getVariable("x")))
        )
        self.assertEqual(ampl.getVariable("x").numInstances(), 1)
        self.assertEqual(ampl.getVariable("y").numInstances(), 2)
        self.assertEqual(ampl.getVariable("z").numInstances(), 4)
        self.assertEqual(ampl.getVariable("xx").numInstances(), 4)
        self.assertEqual(ampl.getVariable("yy").numInstances(), 8)
        self.assertEqual(ampl.getVariable("zz").numInstances(), 8)
        self.assertEqual(
            max(var.numInstances() for name, var in ampl.getVariables()), 8
        )
        self.assertEqual(ampl.getVariable("x").name(), ampl.getVariables()["x"].name())

    def testConstraintIterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.eval("s.t. c_x: x = 0;")
        self.assertEqual(ampl.getConstraints().size(), len(ampl.getConstraints()))
        self.assertEqual(ampl.getConstraints().size(), len(list(ampl.getConstraints())))
        self.assertEqual(ampl.getConstraints().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.getConstraints().size(), 0)
        ampl.eval("var x;")
        ampl.eval("s.t. c_x: x = 0;")
        self.assertEqual(ampl.getConstraints().size(), 1)
        ampl.eval("var y{1..2};")
        ampl.eval("s.t. c_y{i in 1..2}: y[i] = i;")
        self.assertEqual(ampl.getConstraints().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('s.t. c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j] = i;')
        self.assertEqual(ampl.getConstraints().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        ampl.eval("s.t. c_xx{(i, j) in {1..2,4..5}}: xx[i,j] = i+j;")
        self.assertEqual(ampl.getConstraints().size(), 4)
        self.assertEqual(
            ampl.getConstraint("c_x").numInstances(),
            len(dict(ampl.getConstraint("c_x"))),
        )
        self.assertEqual(ampl.getConstraint("c_x").numInstances(), 1)
        self.assertEqual(ampl.getConstraint("c_y").numInstances(), 2)
        self.assertEqual(ampl.getConstraint("c_z").numInstances(), 4)
        self.assertEqual(ampl.getConstraint("c_xx").numInstances(), 4)
        self.assertEqual(
            max(con.numInstances() for name, con in ampl.getConstraints()), 4
        )
        self.assertEqual(
            ampl.getConstraint("c_x").name(), ampl.getConstraints()["c_x"].name()
        )

    def testObjectiveIterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.eval("maximize c_x: x;")
        self.assertEqual(ampl.getObjectives().size(), len(ampl.getObjectives()))
        self.assertEqual(ampl.getObjectives().size(), len(list(ampl.getObjectives())))
        self.assertEqual(ampl.getObjectives().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.getObjectives().size(), 0)
        ampl.eval("var x;")
        ampl.eval("maximize c_x: x;")
        self.assertEqual(ampl.getObjectives().size(), 1)
        ampl.eval("var y{1..2};")
        ampl.eval("maximize c_y{i in 1..2}: y[i];")
        self.assertEqual(ampl.getObjectives().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('maximize c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j];')
        self.assertEqual(ampl.getObjectives().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        ampl.eval("maximize c_xx{(i, j) in {1..2,4..5}}: xx[i,j];")
        self.assertEqual(ampl.getObjectives().size(), 4)
        self.assertEqual(
            ampl.getObjective("c_x").numInstances(), len(dict(ampl.getObjective("c_x")))
        )
        self.assertEqual(ampl.getObjective("c_x").numInstances(), 1)
        self.assertEqual(ampl.getObjective("c_y").numInstances(), 2)
        self.assertEqual(ampl.getObjective("c_z").numInstances(), 4)
        self.assertEqual(ampl.getObjective("c_xx").numInstances(), 4)
        self.assertEqual(
            max(obj.numInstances() for name, obj in ampl.getObjectives()), 4
        )
        self.assertEqual(
            ampl.getObjective("c_x").name(), ampl.getObjectives()["c_x"].name()
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import os
import sys
import ctypes
import platform

if platform.system().startswith(("Windows", "MSYS", "CYGWIN", "MINGW")):
    libbase = os.path.join(os.path.dirname(__file__), "cppinterface", "lib")
    lib32 = os.path.join(libbase, "intel32")
    lib64 = os.path.join(libbase, "amd64")
    from glob import glob

    try:
        if ctypes.sizeof(ctypes.c_voidp) == 4:
            dllfile = glob(lib32 + "/*.dll")[0]
        else:
            dllfile = glob(lib64 + "/*.dll")[0]
        ctypes.CDLL(dllfile)
    except Exception as exp:
        raise exp

sys.path.append(os.path.join(os.path.dirname(__file__), "cppinterface"))
try:
    from amplpython import _READTABLE, _WRITETABLE
    from amplpython import *
except Exception as exp:
    raise exp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import os

from .. import modules
from . import TestBase


class TestLoad(TestBase.TestBase):
    """Test ampltools.modules: load, unload, and path."""

    def test_load_verbose(self):
        self._test_load(verbose=True)

    def test_load_silent(self):
        self._test_load(verbose=False)

    def _test_load(self, verbose):
        self.assertEqual(modules.installed(), [])

        initial_path = self.get_env_path_list()

        self.assertEqual(modules.installed(), [])
        modules.preload(verbose=verbose)
        self.assertEqual(initial_path, self.get_env_path_list())

        modules.install("highs", verbose=verbose)
        self.assertNotEqual(initial_path, self.get_env_path_list())
        self.assertTrue(os.path.isfile(modules.find("highs")))

        modules.load("highs", verbose=verbose)
        self.assertNotEqual(initial_path, self.get_env_path_list())
        expected = modules.path().split(os.pathsep) + initial_path
        self.assertEqual(expected, self.get_env_path_list())
        self.assertEqual(len(initial_path) + 2, len(self.get_env_path_list()))
        self.assertIn(modules.path("highs", add_base=False), self.get_env_path_list())

        modules.install(["cbc"], verbose=verbose)
        modules.unload(["base", "cbc", "highs"])
        modules.load(["cbc", "highs"], verbose=verbose)
        expected = modules.path().split(os.pathsep) + initial_path
        self.assertEqual(expected, self.get_env_path_list())
        self.assertEqual(len(initial_path) + 3, len(self.get_env_path_list()))

        modules.unload("highs")
        self.assertNotEqual(initial_path, self.get_env_path_list())
        self.assertEqual(len(initial_path) + 2, len(self.get_env_path_list()))

        modules.unload(["cbc"])
        self.assertNotEqual(initial_path, self.get_env_path_list())
        self.assertEqual(len(initial_path) + 1, len(self.get_env_path_list()))

        modules.unload("base")
        self.assertEqual(initial_path, self.get_env_path_list())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import os

import amplpy
from . import TestBase


class TestAMPL(TestBase.TestBase):
    """Test AMPL."""

    def test_ampl(self):
        from amplpy import Set, Parameter, Variable, Constraint, Objective

        ampl = self.ampl
        self.assertEqual(ampl.get_data("1..3")._get_num_rows(), 3)
        self.assertEqual(ampl.get_data("1..3")._get_num_cols(), 1)
        ampl.eval("set X := 1..10;")
        self.assertTrue(isinstance(ampl.get_entity("X"), amplpy.Entity))
        self.assertEqual(ampl.get_entity("X").get_values()._get_num_rows(), 10)
        self.assertEqual(ampl.get_data("X")._get_num_rows(), 10)

        with self.assertRaises(RuntimeError):
            ampl.get_data("XXX")
        with self.assertRaises(KeyError):
            ampl.get_entity("XXX")
        with self.assertRaises(KeyError):
            ampl.get_set("XXX")
        with self.assertRaises(KeyError):
            ampl.get_parameter("XXX")
        with self.assertRaises(KeyError):
            ampl.get_variable("XXX")
        with self.assertRaises(KeyError):
            ampl.get_constraint("XXX")
        with self.assertRaises(KeyError):
            ampl.get_objective("XXX")
        self.assertEqual(len(ampl.get_sets()), 1)
        ampl.reset()
        self.assertEqual(len(ampl.get_sets()), 0)
        ampl.eval("set _s; param _p; var _v; s.t. _c: _v = 0; maximize _o: 0;")
        self.assertTrue(isinstance(ampl.get_set("_s"), Set))
        self.assertTrue(isinstance(ampl.get_parameter("_p"), Parameter))
        self.assertTrue(isinstance(ampl.get_variable("_v"), Variable))
        self.assertTrue(isinstance(ampl.get_constraint("_c"), Constraint))
        self.assertTrue(isinstance(ampl.get_objective("_o"), Objective))
        self.assertEqual(len(ampl.get_sets()), 1)
        self.assertEqual(len(ampl.get_parameters()), 1)
        self.assertEqual(len(ampl.get_variables()), 1)
        self.assertEqual(len(ampl.get_constraints()), 1)
        self.assertEqual(len(ampl.get_objectives()), 1)
        ampl.reset()
        with self.assertRaises(ValueError):
            ampl.eval("X")
        self.assertTrue(ampl.is_running())

    def test_getdata_multi(self):
        ampl = self.ampl
        ampl.eval(
            """
        param p1{i in 1..10} := 1*i;
        param p2{i in 1..10} := 2*i;
        param p3{i in 0..10: i >= 1} := 3*i;
        """
        )
        df = ampl.get_data("p1", "p2", "p3")
        df = ampl.get_data("p1", "p2", "p3")
        self.assertEqual(list(df._get_column("p1")), [i for i in range(1, 10 + 1)])
        self.assertEqual(list(df._get_column("p2")), [2 * i for i in range(1, 10 + 1)])
        self.assertEqual(list(df._get_column("p3")), [3 * i for i in range(1, 10 + 1)])
        self.assertEqual(df._get_headers(), ("index0", "p1", "p2", "p3"))
        df = ampl.get_data("1..10", "p1", "p2", "p3")
        self.assertEqual(list(df._get_column("p1")), [i for i in range(1, 10 + 1)])
        self.assertEqual(list(df._get_column("p2")), [2 * i for i in range(1, 10 + 1)])
        self.assertEqual(list(df._get_column("p3")), [3 * i for i in range(1, 10 + 1)])
        self.assertEqual(df._get_headers(), ("1 .. 10", "p1", "p2", "p3"))
        with self.assertRaises(TypeError):
            ampl.get_data("1..11", "p1", "p2", "p3")

    def test_path(self):
        ampl = self.ampl
        self.assertEqual(os.path.abspath(ampl.cd()), os.path.abspath(os.curdir))
        ampl.cd("..")
        self.assertEqual(
            os.path.abspath(ampl.cd()), os.path.abspath(os.path.join(os.curdir, ".."))
        )

    def test_options(self):
        ampl = self.ampl
        ampl.set_option("a", "s")
        ampl.set_option("b", 123)
        ampl.set_option("c", 1.23)
        ampl.set_option("d", True)
        with self.assertRaises(TypeError):
            ampl.set_option("e", None)
        self.assertEqual(ampl.get_option("a"), "s")
        self.assertEqual(ampl.get_option("b"), 123)
        self.assertEqual(ampl.get_option("c"), 1.23)
        self.assertEqual(ampl.get_option("d"), True)

    def test_handlers(self):
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def __init__(self):
                self.lastmsg = None
                super(MyOutputHandler, self).__init__()

            def output(self, kind, msg):
                if kind == amplpy.Kind.DISPLAY:
                    print(f"Display: |{msg}|")
                self.lastmsg = msg

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self):
                self.last_wrror = None
                self.last_warning = None

            def error(self, exception):
                print(type(exception))
                print("Error:", exception.get_message())
                self.last_error = exception

            def warning(self, exception):
                print(type(exception))
                print("Warning:", exception.get_message())
                self.last_warning = exception

            def check(self):
                pass

        output_handler = MyOutputHandler()
        ampl.set_output_handler(output_handler)
        error_handler = MyErrorHandler()
        ampl.set_error_handler(error_handler)
        self.assertEqual(ampl.get_output_handler(), output_handler)
        self.assertEqual(ampl.get_error_handler(), error_handler)
        ampl.display("1", "2", "3")
        self.assertTrue("1 = 1" in output_handler.lastmsg)
        self.assertTrue("2 = 2" in output_handler.lastmsg)
        self.assertTrue("3 = 3" in output_handler.lastmsg)
        ampl.eval("display X;")
        self.assertTrue(
            error_handler.last_warning.get_message().startswith("X is not defined")
        )
        ampl.eval("diy X;")
        self.assertTrue(
            error_handler.last_error.get_message().startswith("syntax error")
        )
        self.assertTrue(isinstance(error_handler.last_warning.get_source_name(), str))
        self.assertTrue(isinstance(error_handler.last_warning.get_line_number(), int))
        self.assertTrue(isinstance(error_handler.last_warning.get_offset(), int))
        self.assertTrue(isinstance(error_handler.last_warning.get_message(), str))

    def test_empty_handlers(self):
        ampl = self.ampl
        output_handler = amplpy.OutputHandler()
        error_handler = amplpy.ErrorHandler()
        ampl.set_output_handler(output_handler)
        ampl.set_error_handler(error_handler)
        ampl.eval("display 1;")

    def test_broken_handlers(self):
        ampl = self.ampl

        class OutputHandlerRaise(amplpy.OutputHandler):
            def output(self, kind, msg):
                assert False

        class ErrorHandlerIgnore(amplpy.ErrorHandler):
            def error(self, exception):
                assert True

            def warning(self, exception):
                assert True

        class ErrorHandlerRaise(amplpy.ErrorHandler):
            def error(self, exception):
                raise RuntimeError("failed")

            def warning(self, exception):
                raise RuntimeError("failed")

        with self.assertRaises(amplpy.AMPLException):
            ampl.eval("X X;")

        error_handler_ignore = ErrorHandlerIgnore()
        ampl.set_error_handler(error_handler_ignore)
        ampl.eval("X X;")

        error_handler_raise = ErrorHandlerRaise()
        ampl.set_error_handler(error_handler_raise)
        with self.assertRaises(RuntimeError):
            ampl.eval("X X;")

        ampl.set_output_handler(OutputHandlerRaise())
        with self.assertRaises(RuntimeError):
            ampl.eval("display 1;")

    def test_get_output(self):
        ampl = self.ampl
        self.assertEqual(ampl.get_output("display 5;"), "5 = 5\n\n")
        with self.assertRaises(ValueError):
            ampl.get_output("display 3")
        with self.assertRaises(ValueError):
            ampl.get_output("for {i in 1..10} {")
        self.assertEqual(ampl.get_output("display 5; display 1;"), "5 = 5\n\n1 = 1\n\n")

    def test_export(self):
        ampl = self.ampl
        model = self.str2file(
            "model.mod",
            """
            set A;
            set FLOOR;
            set family {FLOOR};
        """,
        )
        data = self.str2file(
            "data.dat",
            """
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        """,
        )
        ampl.read(model)
        ampl.read_data(data)
        model2 = self.tmpfile("model2.mod")
        data2 = self.tmpfile("data2.dat")
        ampl.export_model(model2)
        ampl.export_data(data2)

        ampl.reset()
        ampl.read(model2)
        ampl.read_data(data2)
        self.assertEqual(
            ampl.set["family"]["first"].get_values().to_list(), ["Gutierrez"]
        )
        self.assertEqual(
            ampl.set["family"]["second"].get_values().to_list(), ["Montoro"]
        )
        self.assertEqual(ampl.set["A"].get_values().to_list(), [1, 2, 3, 4])

    def test_snapshot(self):
        ampl = self.ampl
        model = self.str2file(
            "model.mod",
            """
            set A;
            set FLOOR;
            set family {FLOOR};
        """,
        )
        data = self.str2file(
            "data.dat",
            """
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        """,
        )
        ampl.read(model)
        ampl.read_data(data)
        snapshot_file = self.tmpfile("snapshot.run")
        ampl.snapshot(snapshot_file)

        ampl.reset()
        ampl.read(snapshot_file)
        self.assertEqual(
            ampl.set["family"]["first"].get_values().to_list(), ["Gutierrez"]
        )
        self.assertEqual(
            ampl.set["family"]["second"].get_values().to_list(), ["Montoro"]
        )
        self.assertEqual(ampl.set["A"].get_values().to_list(), [1, 2, 3, 4])

    def test_export_nofiles(self):
        ampl = self.ampl
        model = self.str2file(
            "model.mod",
            """
            set A;
            set FLOOR;
            set family {FLOOR};
        """,
        )
        data = self.str2file(
            "data.dat",
            """
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        """,
        )
        ampl.read(model)
        ampl.read_data(data)
        model2 = ampl.export_model()
        data2 = ampl.export_data()

        ampl.reset()
        ampl.eval(model2)
        ampl.eval(data2)
        self.assertEqual(
            ampl.set["family"]["first"].get_values().to_list(), ["Gutierrez"]
        )
        self.assertEqual(
            ampl.set["family"]["second"].get_values().to_list(), ["Montoro"]
        )
        self.assertEqual(ampl.set["A"].get_values().to_list(), [1, 2, 3, 4])

    def test_snapshot_nofiles(self):
        ampl = self.ampl
        model = self.str2file(
            "model.mod",
            """
            set A;
            set FLOOR;
            set family {FLOOR};
        """,
        )
        data = self.str2file(
            "data.dat",
            """
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        """,
        )
        ampl.read(model)
        ampl.read_data(data)
        snapshot = ampl.snapshot()
        ampl.reset()

        ampl.eval(snapshot)
        self.assertEqual(
            ampl.set["family"]["first"].get_values().to_list(), ["Gutierrez"]
        )
        self.assertEqual(
            ampl.set["family"]["second"].get_values().to_list(), ["Montoro"]
        )
        self.assertEqual(ampl.set["A"].get_values().to_list(), [1, 2, 3, 4])

    def test_pathlib(self):
        ampl = self.ampl
        try:
            from pathlib import Path
        except ImportError:
            return
        model = self.str2file("model.mod", "set A := 1..10;")
        p = Path(model)
        ampl.read(p)
        ampl.display("A")

    def test_write(self):
        ampl = self.ampl
        ampl.eval(
            """
        var x;
        var y;
        maximize obj: x+y;
        s.t. c1: x <= 0;
        s.t. c2: x >= 1;
        """
        )
        with self.assertRaises(amplpy.InfeasibilityException):
            ampl.write("bmod", "rc")
        ampl.reset()
        ampl.eval(
            """
        var x >= 0;
        var y >= 0;
        maximize obj: x+y;
        s.t. c1: x <= 0;
        s.t. c2: y <= 0;
        """
        )
        with self.assertRaises(amplpy.PresolveException):
            ampl.write("bmod", "rc")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import os
from .. import modules


class TestBase(unittest.TestCase):
    def setUp(self):
        modules.uninstall()

    def get_env_path(self):
        return os.environ["PATH"]

    def get_env_path_list(self):
        return self.get_env_path().split(os.pathsep)

    def tearDown(self):
        modules.uninstall()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-


class BaseClass(object):
    def __init__(self, _impl):
        self._impl = _impl

    def to_string(self):
        return self._impl.toString()

    def __str__(self):
        return self.to_string()

    # Aliases
    toString = to_string



# -*- coding: utf-8 -*-
import platform
import os


def cloud_platform_name():
    """Guesses the name of cloud platform currently running on."""
    if platform.system() != "Linux":
        return None
    envkeys = dict(os.environ).keys()
    if any(key.startswith("COLAB_") for key in envkeys):
        return "colab"
    if os.path.isdir("/content/"):
        return "colab"
    if any(key.startswith("KAGGLE_") for key in envkeys):
        return "kaggle"
    if any(key.startswith("PAPERSPACE_") for key in envkeys):
        return "paperspace"
    if os.path.isdir("/home/studio-lab-user"):
        return "sagemaker-studio-lab"
    return None



# -*- coding: utf-8 -*-
import sys
import os

_parent_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
_vendor_dir = os.path.join(_parent_dir, "vendor")
if _vendor_dir not in sys.path:
    sys.path.append(_vendor_dir)

if __name__ == "__main__":
    try:
        from ampltools.modules import _main
    except ImportError:
        print(
            "Failed to import 'ampltools.modules'. Install or upgrade with:\n"
            "   $ python -m pip install ampltools --upgrade",
            file=sys.stderr,
        )
        sys.exit(1)
    _main()



# -*- coding: utf-8 -*-
from numbers import Real
from collections.abc import Iterable

from .entity import Entity
from .dataframe import DataFrame
from .iterators import MemberRangeIterator

try:
    import numpy as np
except ImportError:
    np = None


class Set(Entity):
    """
    Represents an AMPL set. In case of not indexed sets, this class exposes
    iterators for accessing its elements. The members of the set are tuples.
    All these methods throw a TypeError if called for an indexed set.

    In case of indexed sets, you can gain access to the instances using the
    operator [].

    All the accessors in this class throw an RuntimeError if the instance has
    been deleted in the underlying AMPL interpreter.

    Data can be assigned to the set using the methods
    :func:`~amplpy.Set.set_values`
    (for non-indexed sets only) or using
    :func:`~amplpy.AMPL.set_data`
    and an object of class
    :class:`~amplpy.DataFrame`.
    """

    def __init__(self, _impl):
        Entity.__init__(self, _impl, Set)

    def __setitem__(self, index, value):
        self.__getitem__(index).set_values(value)

    def __iter__(self):
        if self.is_scalar():
            return self.members()
        else:
            return self.instances()

    def instances(self):
        """
        Get an iterator to iterate over all the instances in a Set.
        """
        return Entity.instances(self)

    def arity(self):
        """
        The arity of the set, or number of components in each member of this
        set.
        """
        return int(self._impl.arity())

    def get_values(self):
        """
        Get values of this set in a DataFrame. Valid only for non-indexed sets.
        """
        return Entity.get_values(self)

    def members(self):
        """
        Get members (tuples) of this Set. Valid only for non-indexed sets.
        """
        return MemberRangeIterator(self._impl.members())

    def size(self):
        """
        Get the number of tuples in this set. Valid only for non-indexed sets.
        """
        return int(self._impl.size())

    def contains(self, t):
        """
        Check wether this set instance contains the specified Tuple.
        Valid only for non-indexed sets.

        Args:
            t: Tuple to be found.
        """
        return self._impl.contains(t)

    def set_values(self, values):
        """
        Set the tuples in this set. Valid only for non-indexed sets.

        Args:
            values: A list of tuples or a :class:`~amplpy.DataFrame`.

        In the case of a :class:`~amplpy.DataFrame`, the number of indexing
        columns of the must be equal to the arity of the set. In the case of
        a list of tuples, the arity of each tuple must be equal to the arity
        of the set.

        For example, considering the following AMPL entities and corresponding
        Python objects:

        .. code-block:: ampl

            set A := 1..2;
            param p{i in A} := i+10;
            set AA;

        The following is valid:

        .. code-block:: python

            A, AA = ampl.getSet('A'), ampl.getSet('AA')
            AA.setValues(A.getValues())  # AA has now the members {1, 2}
        """
        if isinstance(values, DataFrame):
            Entity.set_values(self, values)
        elif isinstance(values, Iterable):
            dimen = self.arity()
            if dimen == 1 and all(isinstance(value, str) for value in values):
                if not isinstance(values, (list, tuple)):
                    values = list(values)
                self._impl.setValuesStr(values, len(values))
            elif dimen == 1 and all(isinstance(value, Real) for value in values):
                values = list(map(float, values))
                self._impl.setValuesDbl(values, len(values))
            else:

                def cast_value(value):
                    if isinstance(value, str):
                        return value
                    elif isinstance(value, Real):
                        return float(value)
                    else:
                        raise TypeError("Excepted string or real.")

                def cast_row(row):
                    if isinstance(row, str):
                        return row
                    elif isinstance(row, Real):
                        return float(row)
                    elif isinstance(row, Iterable):
                        return tuple(map(cast_value, row))
                    else:
                        if dimen == 1:
                            raise TypeError("Excepted string or real.")
                        else:
                            raise ValueError(f"Excepted tuple of arity {dimen}.")

                values = [cast_row(row) for row in values]
                if dimen == 1:
                    if any(isinstance(row, tuple) for row in values):
                        raise ValueError(
                            f"Trying to assign tuples to set of arity {dimen}."
                        )
                else:
                    if any(
                        not isinstance(row, tuple) or len(row) != dimen
                        for row in values
                    ):
                        raise ValueError(f"Expected tuples of arity {dimen}.")
                self._impl.setValuesTuples(values, len(values))
        else:
            Entity.set_values(self, values)

    # Aliases
    getValues = get_values
    setValues = set_values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from . import TestBase


class TestProperties(TestBase.TestBase):
    """Test Properties."""

    def test_var_prop(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.var["x"] = 10
        self.assertEqual(ampl.var["x"].value(), ampl.get_variable("x").value())
        self.assertEqual(len(list(ampl.var)), len(list(ampl.get_variables())))

    def test_con_prop(self):
        ampl = self.ampl
        ampl.eval("var x; s.t. c: x = 10;")
        ampl.con["c"] = 10
        self.assertEqual(ampl.con["c"].dual(), ampl.get_constraint("c").dual())
        self.assertEqual(len(list(ampl.con)), len(list(ampl.get_constraints())))

    def test_obj_prop(self):
        ampl = self.ampl
        ampl.eval("var x; maximize obj: x;")
        self.assertEqual(ampl.obj["obj"].name(), ampl.get_objective("obj").name())
        self.assertEqual(len(list(ampl.obj)), len(list(ampl.get_objectives())))

    def test_set_prop(self):
        ampl = self.ampl
        ampl.eval("set s;")
        ampl.set["s"] = [1, 2, 3]
        self.assertEqual(ampl.set["s"].name(), ampl.get_set("s").name())
        self.assertEqual(ampl.set["s"].size(), 3)
        self.assertEqual(len(list(ampl.set)), len(list(ampl.get_sets())))

    def test_param_prop(self):
        ampl = self.ampl
        ampl.eval("param p1; param p2{1..2};")
        self.assertEqual(ampl.param["p1"].name(), ampl.get_parameter("p1").name())
        ampl.param["p1"] = 3
        self.assertEqual(ampl.param["p1"].value(), 3)
        ampl.param["p2"] = {1: 10, 2: 20}
        self.assertEqual(ampl.param["p2"][1], 10)
        self.assertEqual(ampl.param["p2"][2], 20)
        self.assertEqual(len(list(ampl.param)), len(list(ampl.get_parameters())))

    def test_option_prop(self):
        ampl = self.ampl
        ampl.option["solver"] = "gurobi"
        self.assertEqual(ampl.option["solver"], ampl.get_option("solver"))

    def test_solve_result(self):
        ampl = self.ampl
        self.assertEqual(ampl.solve_result, "?")
        self.assertEqual(ampl.solve_result_num, -1)
        ampl.solve()
        self.assertEqual(ampl.solve_result, "solved")
        self.assertEqual(ampl.solve_result_num, 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import os


class TestEnvironment(unittest.TestCase):
    """Test Environment."""

    def test_environment_initialization(self):
        from amplpy import Environment

        env = Environment("binary_directory")
        self.assertEqual("binary_directory", env.get_bin_dir())
        env = Environment("binary_directory", "binary_name")
        self.assertEqual("binary_directory", env.get_bin_dir())
        self.assertEqual("binary_name", env.get_bin_name())
        env.set_bin_dir("binary_directory_2")
        self.assertEqual("binary_directory_2", env.get_bin_dir())
        env.set_bin_name("binary_name_2")
        self.assertEqual("binary_name_2", env.get_bin_name())

    def test_environment(self):
        from amplpy import Environment, AMPL

        env1 = Environment()
        env2 = Environment(os.curdir)
        self.assertEqual(env2.get_bin_dir(), os.curdir)
        env1.set_bin_dir(env2.get_bin_dir())
        self.assertEqual(env1.get_bin_dir(), env1.get_bin_dir())
        self.assertEqual(len(dict(env1)), len(list(env1)))
        self.assertEqual(list(sorted(dict(env1).items())), list(sorted(env1)))
        env1["MyEnvVar"] = "TEST"
        self.assertEqual(env1["MyEnvVar"], "TEST")
        self.assertEqual(env2["MyEnvVar"], None)
        d = dict(env1)
        self.assertEqual(d["MyEnvVar"], "TEST")
        ampl = AMPL(Environment())
        ampl.close()

    def test_libpath(self):
        os.environ["ampl_libpath"] = "abc"
        from amplpy import AMPL, Environment

        ampl = AMPL()
        self.assertEqual(ampl.option["ampl_libpath"], "abc")

        ampl = AMPL(Environment())
        self.assertEqual(ampl.option["ampl_libpath"], "abc")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import amplpy
import os


class TestEnvironment(unittest.TestCase):
    """Test Environment."""

    def testEnvironmentInitialization(self):
        from amplpy import Environment

        env = Environment("binary_directory")
        self.assertEqual("binary_directory", env.getBinDir())
        env = Environment("binary_directory", "binary_name")
        self.assertEqual("binary_directory", env.getBinDir())
        self.assertEqual("binary_name", env.getBinName())
        env.setBinDir("binary_directory_2")
        self.assertEqual("binary_directory_2", env.getBinDir())
        env.setBinName("binary_name_2")
        self.assertEqual("binary_name_2", env.getBinName())

    def testEnvironment(self):
        from amplpy import Environment, AMPL

        env1 = Environment()
        env2 = Environment(os.curdir)
        self.assertEqual(env2.getBinDir(), os.curdir)
        env1.setBinDir(env2.getBinDir())
        self.assertEqual(env1.getBinDir(), env1.getBinDir())
        self.assertEqual(len(dict(env1)), len(list(env1)))
        self.assertEqual(list(sorted(dict(env1).items())), list(sorted(env1)))
        env1["MyEnvVar"] = "TEST"
        self.assertEqual(env1["MyEnvVar"], "TEST")
        self.assertEqual(env2["MyEnvVar"], None)
        d = dict(env1)
        self.assertEqual(d["MyEnvVar"], "TEST")
        ampl = AMPL(Environment())
        ampl.close()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import sys
import os
import tempfile
from .utils import cloud_platform_name


def _normalize_modules(modules=[], add_base=False, skip_base=False):
    prefix = "ampl_module_"
    names = [module.replace("-", "_").replace(prefix, "") for module in modules]
    module_names = [prefix + "base"] if add_base else []
    skip = ["ampl", "base"] if add_base or skip_base else []
    module_names += [prefix + name for name in names if name not in skip]
    return module_names


def installed_modules():
    from pkgutil import iter_modules

    prefix = "ampl_module_"
    installed = []
    for _, name, _ in iter_modules():
        norm = name.replace("-", "_")
        if norm.startswith(prefix):
            installed.append(norm.replace(prefix, ""))

    # Move bundles to the end to load individual modules first
    low_piority = ["coin", "open"]
    for name in low_piority:
        module_name = prefix + name
        if module_name in installed:
            installed.remove(module_name)
            installed.append(module_name)

    return installed


def available_modules():
    from requests import get
    from re import findall

    url = "https://pypi.ampl.com/"
    return [
        l.replace("ampl-module-", "")
        for l in findall(">([^<]+)</a>", get(url).text)
        if l.startswith("ampl-module-")
    ]


def _available_bundles():
    from requests import get
    from json import loads

    url = "https://pypi.ampl.com/bundles.json"
    return loads(get(url).text)["bundles"]


def run_command(cmd, show_output=None, return_output=False, verbose=False):
    """
    Run a system command.
    Args:
        cmd: command to run.
        show_output: show the output of running the command.
        verbose: show verbose output if True.
    """
    from subprocess import check_output, STDOUT, CalledProcessError

    if isinstance(cmd, str):
        shell = True
        cmd_str = cmd
    else:
        shell = False
        cmd_str = " ".join(cmd)
    if verbose:
        print("$ " + cmd_str)
    try:
        output = check_output(cmd, stderr=STDOUT, shell=shell).decode("utf-8")
        if show_output or verbose:
            print(output.rstrip("\n"))
        if return_output:
            return 0, output
        return 0
    except CalledProcessError as e:
        output = e.output.decode("utf-8")
        if verbose:
            print(output.rstrip("\n"))
        if return_output:
            return e.returncode, output
        return e.returncode


def _parse_module(module):
    if "=" not in module:
        return module, ""
    name, version = (
        module[: module.find("=")].strip(" ="),
        "==" + module[module.find("=") + 1 :].strip(" ="),
    )
    return name, version


def install_modules(modules=[], reinstall=False, options=[], verbose=False):
    """
    Install AMPL modules for Python.
    Args:
        modules: list of modules to be installed.
        reinstall: reinstall modules if True.
        options: list of options for pip.
        verbose: show verbose output if True.
    """
    if isinstance(modules, str):
        modules = [modules]
    available = None
    try:
        available = set(available_modules())
    except Exception:
        pass
    bundles = None
    try:
        bundles = _available_bundles()
    except Exception:
        pass
    requirements = modules
    if available:
        requirements = []
        for module in modules:
            version = ""
            if "=" in module:
                module, version = _parse_module(module)
            requirement = module + version
            if requirement in requirements or module == "ampl":
                continue
            elif module in available:
                requirements.append(requirement)
            elif not isinstance(bundles, list):
                requirements.append(requirement)
            else:
                for entry in bundles:
                    bundle, includes = None, []
                    try:
                        bundle = entry["module"]
                        includes = entry["includes"]
                    except:
                        pass
                    if module in includes:
                        raise Exception(
                            f"AMPL module '{module}' is not available. It is included in module '{bundle}'."
                        )
                else:
                    raise Exception(f"AMPL module '{module}' is not available.")

    modules = _normalize_modules(modules=requirements, add_base=True)
    pip_cmd = [sys.executable, "-m", "pip", "install", "-i", "https://pypi.ampl.com"]
    if reinstall:
        pip_cmd += ["--force-reinstall", "--upgrade", "--no-cache"]
    if run_command(pip_cmd + modules + options, verbose=verbose) != 0:
        raise Exception("Failed to install modules.")
    load_modules(modules, verbose=verbose)


def uninstall_modules(modules=[], options=[], verbose=False):
    """
    Uninstall AMPL modules for Python.
    Args:
        modules: list of modules to be installed.
        options: list of options for pip.
        verbose: show verbose output if True.
    """
    if isinstance(modules, str):
        modules = [modules]
    skip_base = True
    installed = installed_modules()
    if modules == [] or "all" in modules:
        skip_base = False
        modules = installed
    if skip_base and "base" in modules:
        if set(modules) != set(installed):
            raise Exception(
                "Base module cannot be uninstalled alone. "
                "You need to uninstall all modules."
            )
        else:
            skip_base = False

    unload_modules(modules)
    modules = _normalize_modules(modules=modules, skip_base=skip_base)
    if len(modules) == 0:
        return
    pip_cmd = [sys.executable, "-m", "pip", "uninstall", "-y"]
    if run_command(pip_cmd + modules + options, verbose=verbose) != 0:
        raise Exception("Failed to uninstall modules.")


def _load_ampl_module(module_name):
    from importlib import import_module

    prefix = "ampl_module_"
    if not module_name.startswith(prefix):
        module_name = prefix + module_name
    module = import_module(module_name)
    try:
        from importlib import reload

        reload(module)
    except:
        pass

    mod_file = module.__file__
    if mod_file is None:
        mod_file = ""
    if mod_file.endswith(".pyc"):
        mod_file = mod_file[:-1]
    if not os.path.isfile(mod_file):
        raise Exception(f"Module {module_name} needs to be reinstalled.")

    bin_dir, version = module.bin_dir, module.__version__
    return bin_dir, version


def _locate_modules(modules, verbose=False):
    path_modules = []
    for module_name in modules:
        if not module_name.startswith("ampl_module_"):
            module_name = "ampl_module_" + module_name
        bin_dir = None
        try:
            bin_dir, _ = _load_ampl_module(module_name)
        except:
            if verbose:
                print(f"Failed to import {module_name}.")
            continue
        if bin_dir not in path_modules:
            path_modules.append(bin_dir)
        if verbose:
            print(f"Imported {module_name}.")

    return path_modules


def _sort_modules_for_loading(modules=[], add_base=True):
    if isinstance(modules, str):
        modules = [modules]
    if len(modules) == 0 or "all" in modules:
        modules = installed_modules()
    modules = [_parse_module(module)[0] for module in modules]
    if not add_base:
        return modules
    return ["base"] + [module for module in modules if module not in ("ampl", "base")]


def generate_requirements(modules=[]):
    """
    Generate requirements.txt content.
    Args:
        modules: list of modules.
        verbose: show verbose output if True.
    """
    modules = _sort_modules_for_loading(modules)
    requirements = "--index-url https://pypi.ampl.com\n"
    requirements += "--extra-index-url https://pypi.org/simple\n"
    for m in modules:
        try:
            _, version = _load_ampl_module(m)
            requirements += f"ampl_module_{m}=={version}\n"
        except:
            pass
    return requirements


def _find_ampl_lic():
    for path in os.environ["PATH"].split(os.pathsep):
        ampl_lic = os.path.abspath(os.path.join(path, "ampl.lic"))
        if os.path.isfile(ampl_lic):
            return ampl_lic
    return None


def _prepare_amplkey_env(verbose=False):
    VAR_AMPL_LICFILE = "AMPL_LICFILE"
    VAR_AMPLKEY_RUNTIME_DIR = "AMPLKEY_RUNTIME_DIR"
    if VAR_AMPL_LICFILE in os.environ and VAR_AMPLKEY_RUNTIME_DIR in os.environ:
        # do nothing if all environment variables are set
        return

    bin_dir, _ = _load_ampl_module("base")
    base_ampl_lic = os.path.abspath(os.path.join(bin_dir, "ampl.lic"))
    if os.access(base_ampl_lic, os.W_OK):
        # do nothing if ampl.lic is writable
        return

    if _find_ampl_lic() != base_ampl_lic:
        # do nothing if ampl.lic is somewhere else
        return

    def set_ampl_lic(ampl_lic):
        if verbose:
            print(f'Setting {VAR_AMPL_LICFILE}="{ampl_lic}".')
        os.environ[VAR_AMPL_LICFILE] = ampl_lic

    def set_amplkey_runtime(amplkey_runtime):
        if verbose:
            print(f'Setting {VAR_AMPLKEY_RUNTIME_DIR}="{amplkey_runtime}".')
        os.environ[VAR_AMPLKEY_RUNTIME_DIR] = amplkey_runtime

    if verbose and VAR_AMPL_LICFILE not in os.environ:
        print(f'Found ampl.lic ("{ampl_lic}") but it is not writable.')
    if VAR_AMPLKEY_RUNTIME_DIR in os.environ:
        amplkey_dir = os.environ[VAR_AMPLKEY_RUNTIME_DIR]
    else:
        try:
            amplkey_dir = os.path.join(os.path.expanduser("~"), ".amplkey")
            if not os.path.isdir(amplkey_dir):
                os.makedirs(amplkey_dir)
        except:
            amplkey_dir = os.path.join(tempfile.gettempdir(), ".amplkey")
            if not os.path.isdir(amplkey_dir):
                os.makedirs(amplkey_dir)

    if VAR_AMPL_LICFILE not in os.environ:
        ampl_lic = os.path.join(amplkey_dir, "ampl.lic")
        if not os.path.isfile(ampl_lic):
            open(ampl_lic, "w").write(open(base_ampl_lic, "r").read())
        set_ampl_lic(ampl_lic)
    if VAR_AMPLKEY_RUNTIME_DIR not in os.environ:
        set_amplkey_runtime(amplkey_dir)


def load_modules(modules=[], head=True, verbose=False):
    """
    Load AMPL modules.
    Args:
        modules: list of modules to be loaded.
        head: add to the head of PATH if True.
        verbose: show verbose output if True.
    """
    path_modules = []
    path_others = []
    for path in os.environ["PATH"].split(os.pathsep):
        if path.endswith("bin") and "ampl_module_" in path:
            path_modules.append(path)
        else:
            path_others.append(path)

    modules = _sort_modules_for_loading(modules)
    for path in _locate_modules(modules, verbose=verbose):
        if path not in path_modules:
            path_modules.append(path)

    if head:
        os.environ["PATH"] = os.pathsep.join(path_modules + path_others)
    else:
        os.environ["PATH"] = os.pathsep.join(path_others + path_modules)

    _prepare_amplkey_env()

    # Add all modules to ampl_libpath (necessary for plugins)
    ampl_libpath = os.environ.get("ampl_libpath", "").split("\n")
    for path in path_modules:
        if path not in ampl_libpath:
            ampl_libpath.append(path)
    if ampl_libpath:
        ampl_libpath = [path for path in ampl_libpath if path]
        os.environ["ampl_libpath"] = "\n".join(ampl_libpath)


def unload_modules(modules=[]):
    """
    Unload AMPL modules.
    Args:
        modules: list of modules to be unloaded.
    """
    modules = _sort_modules_for_loading(modules, add_base=False)
    to_remove = tuple(
        path[path.find("ampl_module_") :] for path in _locate_modules(modules)
    )
    os.environ["PATH"] = os.pathsep.join(
        [
            path
            for path in os.environ["PATH"].split(os.pathsep)
            if not path.endswith(to_remove)
        ]
    )


def preload_modules(silently=True, verbose=False):
    """
    Load all modules to the end of environment variable PATH.
    Args:
        silently: ignore any errors and just return False
        verbose: show verbose output if True.
    """
    try:
        # head = False if _find_ampl_lic() else True
        load_modules(head=True, verbose=verbose)
        return True
    except:
        if not silently:
            raise
        return False


def path(modules=[], add_base=True):
    """
    Return PATH for AMPL modules.
    Args:
        modules: list of modules to be included.
    """
    modules = _sort_modules_for_loading(modules, add_base=add_base)
    return os.pathsep.join(_locate_modules(modules, verbose=False))


def find(filename):
    """
    Find a file in AMPL modules.
    Args:
        filename: name of the file to locate.
    """
    paths = path().split(os.pathsep)
    for p in paths:
        full_path = os.path.join(p, filename)
        if os.path.isfile(full_path):
            return full_path
        full_path = os.path.join(p, filename + ".exe")
        if os.path.isfile(full_path):
            return full_path
    raise FileNotFoundError(f"Could not find {filename} in any AMPL module.")


def _activate_default_license(platform, retry=3):
    from subprocess import getoutput
    from requests import post
    from time import sleep
    from json import loads
    from tempfile import mktemp, mkdtemp

    url = "https://portal.ampl.com/v1/amplkey"
    cmd = find("leasefingerprint") + " -s"
    fp = getoutput(cmd)
    for i in range(1, retry + 1):
        response = post(url, data={"uuid": platform, "fingerprint": fp})
        if response.status_code == 200:
            break
        elif i < retry:
            print("Default license activation failed. Retrying...")
            sleep(i)
        else:
            raise Exception("Failed to retrieve default license")
    tmpfile = mktemp(".lic")
    open(tmpfile, "w").write(loads(response.text)["license"])
    os.environ["AMPL_LICFILE"] = tmpfile
    os.environ["AMPL_LICFILE_DEFAULT"] = tmpfile
    os.environ["AMPLKEY_RUNTIME_DIR"] = mkdtemp()


def _handle_default_uuid():
    if "AMPL_LICFILE" in os.environ:
        return os.environ.get("AMPL_LICFILE", "") == os.environ.get(
            "AMPL_LICFILE_DEFAULT", ""
        )
    if cloud_platform_name() != "colab":
        return False
    try:
        _activate_default_license("colab")
        return True
    except:
        print("Failed to activate default license.")
        return False


def activate_license(uuid, verbose=False):
    """
    Activate an AMPL license using the UUID.
    Args:
        uuid: license uuid.
        verbose: show verbose output if True.
    """
    load_modules()
    if not isinstance(uuid, str):
        raise Exception("The license UUID must be a string.")
    if uuid == "default" or "license-uuid" in uuid:
        if _handle_default_uuid():
            return True
        else:
            print(
                "Please provide a valid license UUID. "
                "You can use free https://ampl.com/ce or https://ampl.com/courses licenses."
            )
            raise Exception(
                "Invalid license UUID. "
                "You can use free https://ampl.com/ce or https://ampl.com/courses licenses."
            )
    exit_code = run_command(
        ["amplkey", "activate", "--uuid", uuid],
        verbose=verbose,
    )
    if exit_code != 0:
        raise Exception("The license activation failed.")
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from . import TestBase
import unittest
import amplpy
import os


class TestAMPL(TestBase.TestBase):
    """Test AMPL."""

    def testAMPL(self):
        from amplpy import Set, Parameter, Variable, Constraint, Objective

        ampl = self.ampl
        self.assertEqual(ampl.getData("1..3")._getNumRows(), 3)
        self.assertEqual(ampl.getData("1..3")._getNumCols(), 1)
        ampl.eval("set X := 1..10;")
        self.assertTrue(isinstance(ampl.getEntity("X"), amplpy.Entity))
        self.assertEqual(ampl.getEntity("X").getValues()._getNumRows(), 10)
        self.assertEqual(ampl.getData("X")._getNumRows(), 10)

        with self.assertRaises(RuntimeError):
            self.assertRaises(ampl.getData("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getEntity("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getSet("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getParameter("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getVariable("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getConstraint("XXX"))
        with self.assertRaises(KeyError):
            self.assertRaises(ampl.getObjective("XXX"))
        ampl.reset()
        ampl.eval("set _s; param _p; var _v; s.t. _c: _v = 0; maximize _o: 0;")
        self.assertTrue(isinstance(ampl.getSet("_s"), Set))
        self.assertTrue(isinstance(ampl.getParameter("_p"), Parameter))
        self.assertTrue(isinstance(ampl.getVariable("_v"), Variable))
        self.assertTrue(isinstance(ampl.getConstraint("_c"), Constraint))
        self.assertTrue(isinstance(ampl.getObjective("_o"), Objective))
        print(list(ampl.getSets()))
        self.assertEqual(len(ampl.getSets()), 1)
        self.assertEqual(len(ampl.getParameters()), 1)
        self.assertEqual(len(ampl.getVariables()), 1)
        self.assertEqual(len(ampl.getConstraints()), 1)
        self.assertEqual(len(ampl.getObjectives()), 1)
        ampl.reset()
        with self.assertRaises(ValueError):
            ampl.eval("X")
        self.assertTrue(ampl.isRunning())

    def testPath(self):
        ampl = self.ampl
        self.assertEqual(os.path.abspath(ampl.cd()), os.path.abspath(os.curdir))
        ampl.cd("..")
        self.assertEqual(
            os.path.abspath(ampl.cd()), os.path.abspath(os.path.join(os.curdir, ".."))
        )

    def testOptions(self):
        ampl = self.ampl
        ampl.setOption("a", "s")
        ampl.setOption("b", 123)
        ampl.setOption("c", 1.23)
        ampl.setOption("d", True)
        with self.assertRaises(TypeError):
            ampl.setOption("e", None)
        self.assertEqual(ampl.getOption("a"), "s")
        self.assertEqual(ampl.getOption("b"), 123)
        self.assertEqual(ampl.getOption("c"), 1.23)
        self.assertEqual(ampl.getOption("d"), True)

    def testHandlers(self):
        from time import sleep

        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def __init__(self):
                self.lastmsg = None

            def output(self, kind, msg):
                if kind == amplpy.Kind.DISPLAY:
                    print(f"Display: |{msg}|")
                self.lastmsg = msg

        class MyErrorHandler(amplpy.ErrorHandler):
            def __init__(self):
                self.lastError = None
                self.lastWarning = None

            def error(self, exception):
                print(type(exception))
                print("Error:", exception.getMessage())
                self.lastError = exception

            def warning(self, exception):
                print(type(exception))
                print("Warning:", exception.getMessage())
                self.lastWarning = exception

            def check(self):
                pass

        outputHandler = MyOutputHandler()
        ampl.setOutputHandler(outputHandler)
        errorHandler = MyErrorHandler()
        ampl.setErrorHandler(errorHandler)
        self.assertEqual(ampl.getOutputHandler(), outputHandler)
        self.assertEqual(ampl.getErrorHandler(), errorHandler)
        ampl.display("1", "2", "3")
        self.assertTrue("1 = 1" in outputHandler.lastmsg)
        self.assertTrue("2 = 2" in outputHandler.lastmsg)
        self.assertTrue("3 = 3" in outputHandler.lastmsg)
        ampl.eval("display X;")
        self.assertTrue(
            errorHandler.lastWarning.getMessage().startswith("X is not defined")
        )
        ampl.eval("diy X;")
        self.assertTrue(errorHandler.lastError.getMessage().startswith("syntax error"))
        self.assertTrue(isinstance(errorHandler.lastWarning.getSourceName(), str))
        self.assertTrue(isinstance(errorHandler.lastWarning.getLineNumber(), int))
        self.assertTrue(isinstance(errorHandler.lastWarning.getOffset(), int))
        self.assertTrue(isinstance(errorHandler.lastWarning.getMessage(), str))

    def testEmptyHandlers(self):
        ampl = self.ampl
        outputHandler = amplpy.OutputHandler()
        errorHandler = amplpy.ErrorHandler()
        ampl.setOutputHandler(outputHandler)
        ampl.setErrorHandler(errorHandler)
        ampl.eval("display 1;")

    def testBrokenHandlers(self):
        ampl = self.ampl

        class MyOutputHandler(amplpy.OutputHandler):
            def output(self, kind, msg):
                assert False

        class ErrorHandlerIgnore(amplpy.ErrorHandler):
            def error(self, exception):
                assert True

            def warning(self, exception):
                assert True

        class ErrorHandlerRaise(amplpy.ErrorHandler):
            def error(self, exception):
                raise RuntimeError("failed")

            def warning(self, exception):
                raise RuntimeError("failed")

        with self.assertRaises(amplpy.AMPLException):
            ampl.eval("X X;")

        errorHandlerIgnore = ErrorHandlerIgnore()
        ampl.setErrorHandler(errorHandlerIgnore)
        ampl.eval("X X;")

        errorHandlerRaise = ErrorHandlerRaise()
        ampl.setErrorHandler(errorHandlerRaise)
        with self.assertRaises(RuntimeError):
            ampl.eval("X X;")

    def testGetOutput(self):
        ampl = self.ampl
        self.assertEqual(ampl.getOutput("display 5;"), "5 = 5\n\n")
        with self.assertRaises(ValueError):
            ampl.getOutput("display 3")
        with self.assertRaises(ValueError):
            ampl.getOutput("for {i in 1..10} {")
        self.assertEqual(ampl.getOutput("display 5; display 1;"), "5 = 5\n\n1 = 1\n\n")

    def testExport(self):
        ampl = self.ampl
        model = self.str2file(
            "model.mod",
            """
            set A;
            set FLOOR;
            set family {FLOOR};
        """,
        )
        data = self.str2file(
            "data.dat",
            """
            set A := 1, 2, 3, 4;
            set FLOOR := 'first' 'second';
            set family['first'] := 'Gutierrez';
            set family['second'] := 'Montoro';
        """,
        )
        ampl.read(model)
        ampl.readData(data)
        model2 = self.tmpfile("model2.mod")
        data2 = self.tmpfile("data2.dat")
        ampl.exportModel(model2)
        ampl.exportData(data2)
        ampl.reset()
        ampl.read(model2)
        ampl.readData(data2)
        self.assertEqual(
            ampl.set["family"]["first"].getValues().toList(), ["Gutierrez"]
        )
        self.assertEqual(ampl.set["family"]["second"].getValues().toList(), ["Montoro"])
        self.assertEqual(ampl.set["A"].getValues().toList(), [1, 2, 3, 4])

    def testPath(self):
        ampl = self.ampl
        try:
            from pathlib import Path
        except ImportError:
            return
        model = self.str2file("model.mod", "set A := 1..10;")
        p = Path(model)
        ampl.read(p)
        ampl.display("A")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from . import TestBase
import unittest
from amplpy import DataFrame
import string
import random
import amplpy
import os


class TestDataFrame(TestBase.TestBase):
    """Test DataFrame."""

    def testDataFrame(self):
        ampl = self.ampl
        # Create first dataframe (for data indexed over NUTR)
        # Add data row by row
        df1 = DataFrame("NUTR", ("n_min", "n_max"))
        df1._addRow(("A", 700, 20000))
        df1._addRow(("B1", 700, 20000))
        df1._addRow(("B2", 700, 20000))
        df1._addRow(("C", 700, 20000))
        df1._addRow(("CAL", 16000, 24000))
        df1._addRow(("NA", 0.0, 50000))

        # Create second dataframe (for data indexed over FOOD)
        # Add column by column
        df2 = DataFrame("FOOD")
        foods = ["BEEF", "CHK", "FISH", "HAM", "MCH", "MTL", "SPG", "TUR"]
        df2._setColumn("FOOD", foods)
        self.assertEqual(list(df2._getColumn("FOOD")), foods)
        contents = [2] * 8
        df2._addColumn("f_min", contents)
        self.assertEqual(list(df2._getColumn("f_min")), contents)
        contents = [10] * 8
        df2._addColumn("f_max", contents)
        self.assertEqual(list(df2._getColumn("f_max")), contents)
        costs = [3.19, 2.59, 2.29, 2.89, 1.89, 1.99, 1.99, 2.49]
        df2._addColumn("cost", costs)
        self.assertEqual(list(df2._getColumn("cost")), costs)
        labels = [random.choice(string.ascii_letters)] * 8
        df2._addColumn("labels", labels)
        self.assertEqual(list(df2._getColumn("labels")), labels)
        df2._addColumn("empty", [])
        self.assertEqual(list(df2._getColumn("empty")), [None] * 8)

        print(df2._getColumn("FOOD"))
        for index in df2._getColumn("FOOD"):
            print(df2._getRow(index))

        # Create third dataframe, to assign data to the AMPL entity
        # param amt{NUTR, FOOD};
        df3 = DataFrame(("NUTR", "FOOD"))
        # Populate the set columns
        nutrWithMultiplicity = [""] * 48
        foodWithMultiplicity = [""] * 48
        i = 0
        for n in range(6):
            for f in range(8):
                print(df1._getRowByIndex(n)[0])
                nutrWithMultiplicity[i] = df1._getRowByIndex(n)[0]
                foodWithMultiplicity[i] = foods[f]
                i += 1
        df3._setColumn("NUTR", nutrWithMultiplicity)
        df3._setColumn("FOOD", foodWithMultiplicity)

        # Populate with all these values
        values = [
            60,
            8,
            8,
            40,
            15,
            70,
            25,
            60,
            10,
            20,
            15,
            35,
            15,
            15,
            25,
            15,
            15,
            20,
            10,
            10,
            15,
            15,
            15,
            10,
            20,
            0,
            10,
            40,
            35,
            30,
            50,
            20,
            295,
            770,
            440,
            430,
            315,
            400,
            370,
            450,
            968,
            2180,
            945,
            278,
            1182,
            896,
            1329,
            1397,
        ]
        df3._addColumn("amt", values)

    def testPandas(self):
        ampl = self.ampl
        try:
            import pandas as pd
        except ImportError:
            return
        df = pd.DataFrame({"a": [1, 2], "b": [3.5, 4]}, index=["x", "y"])
        ampl.eval(
            """
            set S;
            param a{S};
            param b{S};
        """
        )
        ampl.setData(df, "S")
        self.assertEqual(list(ampl.set["S"].members()), ["x", "y"])
        self.assertEqual(ampl.param["a"]["x"], 1)
        self.assertEqual(ampl.param["b"]["y"], 4)

        df2 = pd.DataFrame(
            {
                "a": [10, 20, 30],
            },
            index=["x", "y", "z"],
        )
        df3 = pd.DataFrame({}, index=["xx", "yy"])
        df = DataFrame.fromPandas(df)
        df2 = DataFrame.fromPandas(df2)
        df3 = DataFrame.fromPandas(df3)
        self.assertTrue(isinstance(df.toDict(), dict))
        self.assertTrue(isinstance(df.toList(), list))
        self.assertTrue(isinstance(df.toPandas(), pd.DataFrame))

        self.assertEqual(df.toList()[0][1:], (1, 3.5))
        self.assertEqual(df2.toList()[0], ("x", 10))
        self.assertEqual(df3.toList()[0], "xx")

        self.assertEqual(set(df.toDict().keys()), set(["x", "y"]))
        self.assertEqual(set(df2.toDict().keys()), set(["x", "y", "z"]))
        self.assertEqual(set(df3.toDict().keys()), set(["xx", "yy"]))

        self.assertEqual(df.toDict()["x"], (1, 3.5))
        self.assertEqual(df2.toDict()["x"], 10)
        self.assertEqual(df3.toDict()["xx"], None)

        csv_file = os.path.join(os.path.dirname(__file__), "data.csv")
        p_df = pd.read_csv(csv_file, sep=";", index_col=0)
        df = DataFrame.fromPandas(p_df)
        self.assertTrue(isinstance(df.toDict(), dict))
        self.assertEqual(set(df.toDict().keys()), set([1.0, 2.0, 3.0]))
        self.assertEqual(set(df.toList()[0]), set([1.0, 0.01]))
        self.assertEqual(set(df.toList()[1]), set([2.0, 0.02]))
        self.assertEqual(set(df.toList()[2]), set([3.0, 0.03]))

    def testPandasAdvanced(self):
        ampl = self.ampl
        try:
            import pandas as pd
        except ImportError:
            return
        ampl.eval(
            """
            set someSet := {"First", "Second"};
            param N := 5;
            param x1{someSet, 1..N} default 0;
            param x2{someSet, 1..N} default 0;
            param x3{someSet, 1..N} default 0;
            param x4{someSet, 1..N} default 0;
        """
        )

        df = pd.DataFrame(
            [[1, 2, 3, 4, 5], [6, 7, 8, 9, 0]],
            index=["First", "Second"],
            columns=[1, 2, 3, 4, 5],
        )
        ampl.param["x1"] = pd.DataFrame.from_dict(
            df.stack().to_dict(), orient="index", columns=["v"]
        )
        ampl.param["x2"] = pd.DataFrame(df.stack())
        ampl.param["x3"] = df.stack().to_dict()
        ampl.param["x4"] = df.stack()
        d1 = ampl.param["x1"].getValues().toDict()
        d2 = ampl.param["x2"].getValues().toDict()
        d3 = ampl.param["x3"].getValues().toDict()
        d4 = ampl.param["x4"].getValues().toDict()
        self.assertEqual(d1, d2)
        self.assertEqual(d2, d3)
        self.assertEqual(d3, d4)

    def testPandasNamedColumns(self):
        ampl = self.ampl
        try:
            import pandas as pd
        except ImportError:
            return
        df_unindexed = pd.DataFrame(
            [
                ["Apple", "Red", 3, 1.29],
                ["Apple", "Green", 9, 0.99],
                ["Pear", "Red", 25, 2.59],
                ["Pear", "Green", 26, 2.79],
                ["Lime", "Green", 99, 0.39],
            ],
            columns=["Fruit", "Color", "Count", "Price"],
        )
        # RangeIndex
        self.assertEqual(
            DataFrame.fromPandas(df_unindexed)._getHeaders(),
            ("index0", "Fruit", "Color", "Count", "Price"),
        )

        # MultiIndex
        df_indexed = df_unindexed.set_index(["Fruit", "Color"])
        self.assertEqual(
            DataFrame.fromPandas(
                df_indexed, index_names=["Fruit", "Color"]
            )._getHeaders(),
            ("Fruit", "Color", "Count", "Price"),
        )

        # Index without name
        df = pd.DataFrame(
            [[1, 2, 3, 4, 5], [6, 7, 8, 9, 0]],
            index=["First", "Second"],
            columns=[1, 2, 3, 4, 5],
        )
        self.assertEqual(
            DataFrame.fromPandas(df.stack())._getHeaders(), ("index0", "index1", "0")
        )

    def testNumpy(self):
        ampl = self.ampl
        try:
            import numpy as np
        except ImportError:
            return
        ampl.eval("set X;")
        arr = np.array([1, 2, 3])
        ampl.set["X"] = arr
        self.assertEqual(list(ampl.set["X"].members()), [1, 2, 3])
        ampl.eval("param p{1..3};")
        ampl.param["p"] = arr
        self.assertEqual(ampl.param["p"][2], 2)
        self.assertEqual(DataFrame.fromNumpy(arr).toList(), [1, 2, 3])
        mat = np.array([[1, 2], [3, 4], [5, 6]])
        self.assertEqual(DataFrame.fromNumpy(mat).toList(), [(1, 2), (3, 4), (5, 6)])
        self.assertEqual(DataFrame.fromNumpy(mat[:, 0]).toList(), [1, 3, 5])
        self.assertEqual(DataFrame.fromNumpy(mat[:, 1]).toList(), [2, 4, 6])

    def testDict(self):
        dic = {"aa": "bb", "c": "a"}
        self.assertEqual(dic, DataFrame.fromDict(dic).toDict())
        dic = {1: 2}
        self.assertEqual(dic, DataFrame.fromDict(dic).toDict())
        dic = {1: 2, 3: 4}
        self.assertEqual(dic, DataFrame.fromDict(dic).toDict())
        dic = {2.0: ("a", "b"), 3: ("1", "2")}
        self.assertEqual(dic, DataFrame.fromDict(dic).toDict())
        dic = {(2.0, "c"): ("a", "b"), (3, "a"): ("1", "2")}
        self.assertEqual(dic, DataFrame.fromDict(dic).toDict())
        df = DataFrame("x", "y")
        dic = {1: 12, 2: 23}
        df._setValues(dic)
        self.assertEqual(dic, df.toDict())
        df = DataFrame("x", ["y", "z"])
        dic = {1: (12, 2), 2: (23, -1)}
        df._setValues(dic)
        self.assertEqual(dic, df.toDict())
        df = DataFrame("x", ["y", "z"])
        df._setValues({1: [1, 2]})
        self.assertEqual({1: (1, 2)}, df.toDict())

    def testNoIndex(self):
        df = DataFrame([], ["x", "y"])
        x = [1, 2, 3]
        y = [4, 5, 6]
        df._setColumn("x", x)
        df._setColumn("y", y)
        with self.assertRaises(ValueError):
            df.toDict()
        pd_df = df.toPandas()
        self.assertEqual(list(pd_df["x"]), x)
        self.assertEqual(list(pd_df["y"]), y)

    def testIter(self):
        df = DataFrame("x", "y")
        dic = {1: 12, 2: 23}
        df._setValues(dic)
        for row in df:
            self.assertTrue(tuple(row) in [(1, 12), (2, 23)])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import sys
import os

from .base import BaseClass
from .outputhandler import OutputHandler, Kind
from .errorhandler import ErrorHandler
from .exceptions import AMPLException, PresolveException, InfeasibilityException
from .iterators import EntityMap
from .objective import Objective
from .variable import Variable
from .constraint import Constraint
from .set import Set
from .parameter import Parameter
from .entity import Entity
from .dataframe import DataFrame, Row, Column
from .utils import add_to_path, multidict, register_magics
from .environment import Environment
from .ampl import AMPL

_parent_dir = os.path.abspath(os.path.dirname(__file__))
_vendor_dir = os.path.join(_parent_dir, "vendor")
if _vendor_dir not in sys.path:
    sys.path.append(_vendor_dir)

try:
    from ampltools import register_magics
except Exception:
    pass
try:
    from ampltools import ampl_notebook
except Exception:
    pass
try:
    from ampltools import add_to_path
except Exception:
    pass
try:
    from . import modules

    modules.preload()
except Exception:
    pass

__version__ = "0.13.0b0"


def _list_aliases():
    from inflection import camelize

    classes = [
        BaseClass,
        OutputHandler,
        ErrorHandler,
        AMPLException,
        EntityMap,
        Runnable,
        Entity,
        Objective,
        Variable,
        Constraint,
        Set,
        Parameter,
        Row,
        Column,
        DataFrame,
        Environment,
        AMPL,
    ]
    for cls in classes:
        print(cls)
        for method in list(dir(cls)):
            if method.startswith("__"):
                continue
            cammel_method = camelize(method, False)
            if cammel_method != method:
                print(f"\t{cammel_method} = {method}")
                # setattr(cls, cammel_method, getattr(cls, method))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from . import TestBase


class TestIterators(TestBase.TestBase):
    """Test Iterators."""

    def test_set_iterators(self):
        ampl = self.ampl
        ampl.eval("set x;")
        self.assertEqual(len(ampl.get_sets()), ampl.get_sets().size())
        self.assertEqual(ampl.get_sets().size(), len(list(ampl.get_sets())))
        self.assertEqual(ampl.get_sets().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.get_sets().size(), 0)
        ampl.eval("set x;")
        self.assertEqual(ampl.get_sets().size(), 1)
        ampl.eval("set y{1..2};")
        self.assertEqual(ampl.get_sets().size(), 2)
        ampl.eval('set z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_sets().size(), 3)
        ampl.eval("set xx{1..2,4..5};")
        self.assertEqual(ampl.get_sets().size(), 4)
        ampl.eval("set yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.get_sets().size(), 5)
        ampl.eval('set zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_sets().size(), 6)
        self.assertEqual(list(ampl.get_set("x").members()), list(ampl.get_set("x")))
        self.assertEqual(ampl.get_set("x").num_instances(), 1)
        self.assertEqual(ampl.get_set("y").num_instances(), 2)
        self.assertEqual(list(ampl.set["y"][1].members()), list(ampl.set["y"][1]))
        self.assertEqual(ampl.get_set("z").num_instances(), 4)
        self.assertEqual(
            list(ampl.set["z"][1, "a"].members()), list(ampl.set["z"][1, "a"])
        )
        self.assertEqual(ampl.get_set("xx").num_instances(), 4)
        self.assertEqual(
            list(ampl.set["xx"][1, 4].members()), list(ampl.set["xx"][1, 4])
        )
        self.assertEqual(ampl.get_set("yy").num_instances(), 8)
        self.assertEqual(
            list(ampl.set["yy"][1, 4, 7].members()), list(ampl.set["yy"][1, 4, 7])
        )
        self.assertEqual(ampl.get_set("zz").num_instances(), 8)
        self.assertEqual(
            list(ampl.set["zz"][1, "a", 5].members()), list(ampl.set["zz"][1, "a", 5])
        )
        self.assertEqual(max(s.num_instances() for name, s in ampl.get_sets()), 8)
        self.assertEqual(ampl.get_set("x").name(), ampl.get_sets()["x"].name())
        with self.assertRaises(KeyError):
            ampl.get_set("notfound")
        with self.assertRaises(KeyError):
            ampl.get_sets()["notfound"]

    def test_parameter_iterators(self):
        ampl = self.ampl
        ampl.eval("param x;")
        self.assertEqual(len(ampl.get_parameters()), ampl.get_parameters().size())
        self.assertEqual(ampl.get_parameters().size(), len(list(ampl.get_parameters())))
        self.assertEqual(ampl.get_parameters().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.get_parameters().size(), 0)
        ampl.eval("param x;")
        self.assertEqual(ampl.get_parameters().size(), 1)
        ampl.eval("param y{1..2};")
        self.assertEqual(ampl.get_parameters().size(), 2)
        ampl.eval('param z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_parameters().size(), 3)
        ampl.eval("param xx{1..2,4..5};")
        self.assertEqual(ampl.get_parameters().size(), 4)
        ampl.eval("param yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.get_parameters().size(), 5)
        ampl.eval('param zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_parameters().size(), 6)
        self.assertEqual(
            ampl.get_parameter("x").num_instances(), len(dict(ampl.get_parameter("x")))
        )
        self.assertEqual(ampl.get_parameter("x").num_instances(), 1)
        self.assertEqual(ampl.get_parameter("y").num_instances(), 2)
        self.assertEqual(ampl.get_parameter("z").num_instances(), 4)
        self.assertEqual(ampl.get_parameter("xx").num_instances(), 4)
        self.assertEqual(ampl.get_parameter("yy").num_instances(), 8)
        self.assertEqual(ampl.get_parameter("zz").num_instances(), 8)
        self.assertEqual(max(p.num_instances() for name, p in ampl.get_parameters()), 8)
        self.assertEqual(
            ampl.get_parameter("x").name(), ampl.get_parameters()["x"].name()
        )
        with self.assertRaises(KeyError):
            ampl.get_parameter("notfound")
        with self.assertRaises(KeyError):
            ampl.get_parameters()["notfound"]

    def test_variable_iterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        self.assertEqual(len(ampl.get_variables()), ampl.get_variables().size())
        self.assertEqual(ampl.get_variables().size(), len(list(ampl.get_variables())))
        self.assertEqual(ampl.get_variables().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.get_variables().size(), 0)
        ampl.eval("var x;")
        self.assertEqual(ampl.get_variables().size(), 1)
        ampl.eval("var y{1..2};")
        self.assertEqual(ampl.get_variables().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        self.assertEqual(ampl.get_variables().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        self.assertEqual(ampl.get_variables().size(), 4)
        ampl.eval("var yy{1..2,4..5,7..8};")
        self.assertEqual(ampl.get_variables().size(), 5)
        ampl.eval('var zz{1..2,{"a", "b"}, 5..6};')
        self.assertEqual(ampl.get_variables().size(), 6)
        self.assertEqual(
            ampl.get_variable("x").num_instances(), len(dict(ampl.get_variable("x")))
        )
        self.assertEqual(ampl.get_variable("x").num_instances(), 1)
        self.assertEqual(ampl.get_variable("y").num_instances(), 2)
        self.assertEqual(ampl.get_variable("z").num_instances(), 4)
        self.assertEqual(ampl.get_variable("xx").num_instances(), 4)
        self.assertEqual(ampl.get_variable("yy").num_instances(), 8)
        self.assertEqual(ampl.get_variable("zz").num_instances(), 8)
        self.assertEqual(
            max(var.num_instances() for name, var in ampl.get_variables()), 8
        )
        self.assertEqual(
            ampl.get_variable("x").name(), ampl.get_variables()["x"].name()
        )
        with self.assertRaises(KeyError):
            ampl.get_variable("notfound")
        with self.assertRaises(KeyError):
            ampl.get_variables()["notfound"]

    def test_constraint_iterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.eval("s.t. c_x: x = 0;")
        self.assertEqual(ampl.get_constraints().size(), len(ampl.get_constraints()))
        self.assertEqual(
            ampl.get_constraints().size(), len(list(ampl.get_constraints()))
        )
        self.assertEqual(ampl.get_constraints().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.get_constraints().size(), 0)
        ampl.eval("var x;")
        ampl.eval("s.t. c_x: x = 0;")
        self.assertEqual(ampl.get_constraints().size(), 1)
        ampl.eval("var y{1..2};")
        ampl.eval("s.t. c_y{i in 1..2}: y[i] = i;")
        self.assertEqual(ampl.get_constraints().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('s.t. c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j] = i;')
        self.assertEqual(ampl.get_constraints().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        ampl.eval("s.t. c_xx{(i, j) in {1..2,4..5}}: xx[i,j] = i+j;")
        self.assertEqual(ampl.get_constraints().size(), 4)
        self.assertEqual(
            ampl.get_constraint("c_x").num_instances(),
            len(dict(ampl.get_constraint("c_x"))),
        )
        self.assertEqual(ampl.get_constraint("c_x").num_instances(), 1)
        self.assertEqual(ampl.get_constraint("c_y").num_instances(), 2)
        self.assertEqual(ampl.get_constraint("c_z").num_instances(), 4)
        self.assertEqual(ampl.get_constraint("c_xx").num_instances(), 4)
        self.assertEqual(
            max(con.num_instances() for name, con in ampl.get_constraints()), 4
        )
        self.assertEqual(
            ampl.get_constraint("c_x").name(), ampl.get_constraints()["c_x"].name()
        )
        with self.assertRaises(KeyError):
            ampl.get_constraint("notfound")
        with self.assertRaises(KeyError):
            ampl.get_constraints()["notfound"]

    def test_objective_iterators(self):
        ampl = self.ampl
        ampl.eval("var x;")
        ampl.eval("maximize c_x: x;")
        self.assertEqual(ampl.get_objectives().size(), len(ampl.get_objectives()))
        self.assertEqual(ampl.get_objectives().size(), len(list(ampl.get_objectives())))
        self.assertEqual(ampl.get_objectives().size(), 1)
        ampl.eval("reset;")
        self.assertEqual(ampl.get_objectives().size(), 0)
        ampl.eval("var x;")
        ampl.eval("maximize c_x: x;")
        self.assertEqual(ampl.get_objectives().size(), 1)
        ampl.eval("var y{1..2};")
        ampl.eval("maximize c_y{i in 1..2}: y[i];")
        self.assertEqual(ampl.get_objectives().size(), 2)
        ampl.eval('var z{1..2, {"a", "b"}};')
        ampl.eval('maximize c_z{(i, j) in {1..2, {"a", "b"}}}: z[i,j];')
        self.assertEqual(ampl.get_objectives().size(), 3)
        ampl.eval("var xx{1..2,4..5};")
        ampl.eval("maximize c_xx{(i, j) in {1..2,4..5}}: xx[i,j];")
        self.assertEqual(ampl.get_objectives().size(), 4)
        self.assertEqual(
            ampl.get_objective("c_x").num_instances(),
            len(dict(ampl.get_objective("c_x"))),
        )
        self.assertEqual(ampl.get_objective("c_x").num_instances(), 1)
        self.assertEqual(ampl.get_objective("c_y").num_instances(), 2)
        self.assertEqual(ampl.get_objective("c_z").num_instances(), 4)
        self.assertEqual(ampl.get_objective("c_xx").num_instances(), 4)
        self.assertEqual(
            max(obj.num_instances() for name, obj in ampl.get_objectives()), 4
        )
        self.assertEqual(
            ampl.get_objective("c_x").name(), ampl.get_objectives()["c_x"].name()
        )
        with self.assertRaises(KeyError):
            ampl.get_objective("notfound")
        with self.assertRaises(KeyError):
            ampl.get_objectives()["notfound"]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import os

from .. import modules
from . import TestBase


class TestRun(TestBase.TestBase):
    """Test ampltools.modules: run, and activate."""

    def test_run_verbose(self):
        self._test_run(verbose=True)

    def test_run_silent(self):
        self._test_run(verbose=False)

    def _test_run(self, verbose):
        uuid = os.environ.get("AMPLKEY_UUID", None)
        if uuid is None:
            raise unittest.SkipTest("Needs AMPLKEY_UUID to run.")

        modules.install("base", verbose=verbose)
        modules.load("base", verbose=verbose)

        modules.activate(uuid, verbose=verbose)
        self.assertEqual(
            modules.run(f"amplkey activate --uuid {uuid}", verbose=verbose), 0
        )
        self.assertEqual(
            modules.run(["amplkey", "activate", "--uuid", uuid], verbose=verbose),
            0,
        )

    def test_error_verbose(self):
        self._test_error(verbose=True)

    def test_error_silent(self):
        self._test_error(verbose=False)

    def _test_error(self, verbose):
        modules.install("base", verbose=verbose)
        modules.load("base", verbose=verbose)

        with self.assertRaises(Exception) as context:
            modules.activate("invalid-uuid", verbose=verbose)
        self.assertTrue("The license activation failed." in str(context.exception))

        self.assertNotEqual(
            modules.run("amplkey activate --uuid invalid-uuid", verbose=verbose), 0
        )
        self.assertNotEqual(
            modules.run(
                ["amplkey", "activate", "--uuid", "invalid-uuid"], verbose=verbose
            ),
            0,
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from .test_preload import TestPreload as Test1
from .test_install import TestInstall as Test2
from .test_load import TestLoad as Test3
from .test_run import TestRun as Test4

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
# AMPL Python Tools

This package includes tools to use with [AMPL](https://ampl.com) and [amplpy](https://amplpy.readthedocs.io).

## Links

* GitHub Repository: https://github.com/ampl/amplpy/
* PyPI Repository: https://pypi.python.org/pypi/ampltools
"""
from setuptools import setup
import os


def ls_dir(base_dir):
    """List files recursively."""
    return [
        os.path.join(dirpath.replace(base_dir, "", 1), f)
        for (dirpath, dirnames, files) in os.walk(base_dir)
        for f in files
    ]


setup(
    name="ampltools",
    version="0.7.1",
    description="AMPL Python Tools",
    long_description=__doc__,
    long_description_content_type="text/markdown",
    license="BSD-3",
    platforms="any",
    author="Filipe Brandão",
    author_email="fdabrandao@ampl.com",
    url="http://ampl.com/",
    download_url="https://github.com/ampl/amplpy/tree/master/ampltools",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Topic :: Software Development",
        "Topic :: Scientific/Engineering",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    install_requires=open("requirements.txt").read().split("\n"),
    packages=["ampltools"],
    package_data={"": ls_dir("ampltools/")},
)



# -*- coding: utf-8 -*-
from .amplpypi import (
    install_modules,
    uninstall_modules,
    installed_modules,
    available_modules,
    generate_requirements,
    load_modules,
    path,
    find,
    activate_license,
)
import subprocess
import sys

ERROR = """
Invalid command.
Valid commands:
    install, uninstall, list/installed, available, requirements, path,
    run, and activate.
"""
USAGE = """Usage:
- Install modules:
    $ python -m $PACKAGE.modules install <solver 1> <solver 2> ...
    Example:
        $ python -m $PACKAGE.modules install highs gurobi

- Uninstall modules:
    $ python -m $PACKAGE.modules uninstall <solver 1> <solver 2> ...
    Example:
        $ python -m $PACKAGE.modules uninstall highs gurobi
        $ python -m $PACKAGE.modules uninstall # uninstall all modules

- List installed modules:
    $ python -m $PACKAGE.modules installed

- List modules available to be installed:
    $ python -m $PACKAGE.modules available

- Value to append to the environment variable PATH to access modules
    $ python -m $PACKAGE.modules path
    Example:
        $ export PATH=$PATH:`python -m amplpy.modules path`

- Find the path to a file in any module
    $ python -m $PACKAGE.modules find <file name>
    Example:
        $ python -m amplpy.modules find gurobi

- Generate requirements.txt for the modules currently installed
    $ python -m $PACKAGE.modules requirements

- Run command in the same environment as the modules:
    $ python -m $PACKAGE.modules run <command>
    Example:
        $ python -m $PACKAGE.modules run ampl -v

- Activate a license using amplkey:
    $ python -m $PACKAGE.modules activate <license-uuid>
"""


def _main():
    try:
        _commands(sys.argv)
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)


def verbose_option(options):
    for opt in options:
        if opt.startswith("--"):
            if opt == "--quiet":
                return False
        elif opt.startswith("-"):
            if "q" in opt:
                return False
    return True


def _commands(args):
    usage = USAGE.replace("$PACKAGE", "amplpy" if "amplpy" in args[0] else "ampltools")
    if len(args) < 2:
        print(usage)
        return
    command, args = args[1].lower(), args[2:]
    if command == "usage":
        print(usage)
    elif command == "install":
        modules = [m for m in args if not m.startswith("-")]
        options = [o for o in args if o.startswith("-")]
        install_modules(
            modules=modules, options=options, verbose=verbose_option(options)
        )
    elif command == "uninstall":
        modules = [m for m in args if not m.startswith("-")]
        options = [o for o in args if o.startswith("-")]
        uninstall_modules(
            modules=modules, options=options, verbose=verbose_option(options)
        )
    elif command in ("list", "installed"):
        names = installed_modules()
        if names == []:
            raise Exception("Could not find any modules installed.")
        print("You have the following modules installed:")
        for name in sorted(set(names)):
            print("\t" + name)
    elif command == "available":
        names = available_modules()
        if names == []:
            raise Exception("Could not find any modules for download.")
        print("You can install any of the following modules:")
        for name in sorted(set(names)):
            print("\t" + name)
    elif command == "path":
        modules = [m for m in args if not m.startswith("-")]
        print(path(modules))
    elif command == "find":
        if len(args) != 1:
            raise Exception(ERROR + usage)
        filename = args[0]
        print(find(filename))
    elif command == "run":
        if len(args) == 0:
            raise Exception(ERROR + usage)
        load_modules()
        p = subprocess.run(" ".join(args), shell=True)
        if p.returncode != 0:
            raise Exception(f"Exit code {p.returncode}")
    elif command == "activate":
        if len(args) != 1:
            raise Exception(ERROR + usage)
        uuid = args[0]
        activate_license(uuid, verbose=True)
    elif command == "requirements":
        modules = [m for m in args if not m.startswith("-")]
        print(generate_requirements(modules))
    else:
        raise Exception(ERROR + usage)



# -*- coding: utf-8 -*-
import os
from uuid import UUID


def _is_valid_uuid(uuid):
    if uuid in (None, ""):
        return False
    try:
        UUID(str(uuid))
    except ValueError:
        return False

    return True


def add_to_path(path, head=True):
    if head:
        os.environ["PATH"] = path + os.pathsep + os.environ["PATH"]
    else:
        os.environ["PATH"] = os.environ["PATH"] + os.pathsep + path


def register_magics(store_name="_ampl_cells", ampl_object=None, globals_=None):
    """
    Register jupyter notebook magics ``%%ampl`` and ``%%ampl_eval``.
    Args:
        store_name: Name of the store where ``%%ampl cells`` will be stored.
        ampl_object: Object used to evaluate ``%%ampl_eval`` cells.
    """
    from IPython.core.magic import Magics, magics_class, cell_magic, line_magic
    from IPython import get_ipython

    @magics_class
    class StoreAMPL(Magics):
        def __init__(self, shell=None, **kwargs):
            Magics.__init__(self, shell=shell, **kwargs)
            self._store = []
            shell.user_ns[store_name] = self._store

        @cell_magic
        def ampl(self, line, cell):
            """Store the cell in the store"""
            self._store.append(cell)

        @cell_magic
        def ampl_eval(self, line, cell):
            """Evaluate the cell"""
            if globals_ is not None and isinstance(ampl_object, str):
                ampl = globals_[ampl_object]
            else:
                ampl = ampl_object
            ampl.eval("\n" + cell)

        @line_magic
        def get_ampl(self, line):
            """Retrieve the store"""
            return self._store

    get_ipython().register_magics(StoreAMPL)



# -*- coding: utf-8 -*-
import os
import sys
import shutil
import subprocess

USAGE = """Usage:
    $ python -m $PACKAGE.bundle script.py <extra arguments>
"""


def bundle(args):
    from amplpy import modules, __file__ as amplpy_dir

    main_script = args[0]
    extra_arguments = args[1:]
    basename = os.path.basename(main_script).replace(".py", "")
    dist_dir = os.path.join(os.path.abspath(os.curdir), "dist", basename)

    if os.path.isdir(dist_dir):
        print(f"Deleting: {dist_dir}")
        shutil.rmtree(dist_dir)

    # List of modules and paths to include
    pymods = ["amplpy", "ampltools"] + [
        "ampl_module_" + mod for mod in modules.installed()
    ]
    paths = os.path.join(os.path.dirname(amplpy_dir), "amplpython", "cppinterface")

    # Build PyInstaller command
    cmd = [sys.executable, "-m", "PyInstaller", main_script]
    for pymod in pymods:
        for collect in [
            "--collect-all",
            # "--copy-metadata",
            # "--collect-submodules",
            # "--collect-datas",
            # "--collect-binaries",
        ]:
            cmd += [collect, pymod]
    cmd += ["--paths", paths]
    cmd += extra_arguments

    # Run the command
    print("$ " + " ".join(cmd))
    process = subprocess.Popen(
        cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True
    )
    for line in process.stdout:
        print(line, end="")
    exit_code = process.wait()

    if exit_code == 0:
        # Locate the executable
        executable = os.path.join(dist_dir, basename)
        if not os.path.exists(executable):
            executable += ".exe"
        if os.path.exists(executable):
            print(f'\n\nYour executable is at "{executable}".')

    sys.exit(exit_code)


def _main():
    args = sys.argv
    if len(args) < 2 or not args[1].endswith(".py"):
        print(
            USAGE.replace("$PACKAGE", "amplpy" if "amplpy" in args[0] else "ampltools")
        )
        sys.exit(1)
    bundle(args[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from .. import modules
from . import TestBase


class TestPreload(TestBase.TestBase):
    """Test ampltools.modules: preload."""

    def test_preload_verbose(self):
        self._test_preload(verbose=True)

    def test_preload_silent(self):
        self._test_preload(verbose=False)

    def _test_preload(self, verbose):
        """When preloading without ampl_module_base it is not loaded
        afterwards unless we reload modules.
        """
        modules.unload()
        initial_path = self.get_env_path_list()
        modules.preload(verbose=verbose)
        self.assertEqual(modules.installed(), [])
        requirements = modules.requirements().strip().split("\n")
        self.assertEqual(len(requirements), 2)

        with self.assertRaises(Exception) as context:
            modules.load("cbc", verbose=verbose)
        self.assertTrue(
            "Module ampl_module_base needs to be reinstalled." in str(context.exception)
        )

        modules.install(["cbc"], verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base", "cbc"})
        modules.load()
        self.assertTrue(
            any("ampl_module_cbc" in path for path in self.get_env_path_list())
        )
        self.assertTrue(
            any("ampl_module_base" in path for path in self.get_env_path_list())
        )

        self.assertEqual(len(initial_path) + 2, len(self.get_env_path_list()))

        requirements = modules.requirements().strip().split("\n")
        self.assertEqual(len(requirements), 4)

        self.assertTrue(
            any(req.startswith("ampl_module_base==") for req in requirements[2:])
        )
        self.assertTrue(
            any(req.startswith("ampl_module_cbc==") for req in requirements[2:])
        )

        modules.uninstall(["cbc"], verbose=verbose)
        self.assertEqual(set(modules.installed()), {"base"})

        self.assertEqual(len(initial_path) + 1, len(self.get_env_path_list()))

        requirements = modules.requirements().strip().split("\n")
        self.assertEqual(len(requirements), 3)

        self.assertTrue(
            any(req.startswith("ampl_module_base==") for req in requirements[2:])
        )
        self.assertFalse(
            any(req.startswith("ampl_module_cbc==") for req in requirements[2:])
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import amplpy
from . import TestBase

try:
    import numpy as np
except ImportError:
    np = None
try:
    import pandas as pd
except ImportError:
    pd = None


def load_diet_model(ampl):
    ampl.eval(
        """
        set FOOD ;
        set NUTR ;
        # Parameters
        param cost { FOOD } > 0;
        param f_min { FOOD } >= 0;
        param f_max {j in FOOD } >= f_min [j];
        param n_min { NUTR } >= 0;
        param n_max {i in NUTR } >= n_min [i];
        param amt {NUTR , FOOD } >= 0;
        # Variables
        var Buy {j in FOOD } >= f_min [j], <= f_max [j];
        # Objective
        minimize total_cost : sum {j in FOOD } cost [j] * Buy[j];
        # Contraints
        subject to diet {i in NUTR }:
            n_min [i] <= sum {j in FOOD } amt[i,j] * Buy[j] <= n_max [i];
    """
    )
    ampl.eval(
        """
        data;
        set NUTR := A C B1 B2 NA CAL;
        set FOOD := BEEF CHK FISH HAM MCH MTL SPG TUR ;
        param : cost f_min f_max :=
        BEEF 3.19 2 10
        CHK 2.59 2 10
        FISH 2.29 2 10
        HAM 2.89 2 10
        MCH 1.89 2 10
        MTL 1.99 2 10
        SPG 1.99 2 10
        TUR 2.49 2 10 ;
        param : n_min n_max :=
        A 700 20000
        C 700 20000
        B1 700 20000
        B2 700 20000
        NA 0 50000
        CAL 16000 24000 ;
        param amt (tr):
        A C B1 B2 NA CAL :=
        BEEF 60 20 10 15 938 295
        CHK 8 0 20 20 2180 770
        FISH 8 10 15 10 945 440
        HAM 40 40 35 10 278 430
        MCH 15 35 15 15 1182 315
        MTL 70 30 15 15 896 400
        SPG 25 50 25 15 1329 370
        TUR 60 20 15 10 1397 450 ;
    """
    )


class TestEntities(TestBase.TestBase):
    """Test entities."""

    def test_variable(self):
        load_diet_model(self.ampl)
        ampl = self.ampl
        # ampl.solve()
        self.assertEqual(
            ampl.get_variable("Buy").num_instances(), ampl.get_set("FOOD").size()
        )
        f_min = ampl.get_parameter("f_min")
        f_max = ampl.get_parameter("f_max")
        buy = ampl.get_variable("Buy")
        buy["BEEF"] = 10
        self.assertEqual(buy["BEEF"].value(), 10)
        self.assertTrue(isinstance(buy.get_values(), amplpy.DataFrame))
        self.assertTrue(isinstance(buy.get_values(["val"]), amplpy.DataFrame))
        self.assertTrue(isinstance(buy.get_values("val"), amplpy.DataFrame))
        for index, var in ampl.get_variable("Buy"):
            self.assertTrue(isinstance(var.value(), float))
            var.set_value(f_min[index])
            self.assertEqual(var.value(), f_min[index])
            var.fix()
            self.assertEqual(var.astatus(), "fix")
            var.unfix()
            self.assertEqual(var.astatus(), "in")
            var.fix(f_max[index])
            self.assertEqual(var.value(), f_max[index])
            self.assertTrue(isinstance(var.defeqn(), int))
            self.assertTrue(isinstance(var.dual(), float))
            self.assertTrue(isinstance(var.init(), float))
            self.assertTrue(isinstance(var.init0(), float))
            self.assertTrue(isinstance(var.lb(), float))
            self.assertGreaterEqual(var.lb(), f_min[index])
            self.assertTrue(isinstance(var.ub(), float))
            self.assertLessEqual(var.ub(), f_max[index])
            self.assertTrue(isinstance(var.lb0(), float))
            self.assertEqual(var.lb0(), f_min[index])
            self.assertTrue(isinstance(var.ub0(), float))
            self.assertEqual(var.ub0(), f_max[index])
            self.assertTrue(isinstance(var.lb1(), float))
            self.assertGreaterEqual(var.lb1(), f_min[index])
            self.assertTrue(isinstance(var.ub1(), float))
            self.assertLessEqual(var.ub1(), f_max[index])
            self.assertTrue(isinstance(var.lb2(), float))
            self.assertGreaterEqual(var.lb2(), f_min[index])
            self.assertTrue(isinstance(var.ub2(), float))
            self.assertLessEqual(var.ub2(), f_max[index])
            self.assertTrue(isinstance(var.lrc(), float))
            self.assertTrue(isinstance(var.urc(), float))
            self.assertTrue(isinstance(var.lslack(), float))
            self.assertTrue(isinstance(var.uslack(), float))
            self.assertTrue(isinstance(var.rc(), float))
            self.assertTrue(isinstance(var.slack(), float))
            self.assertTrue(isinstance(var.sstatus(), str))
            self.assertTrue(isinstance(var.status(), str))

    def test_variable_numpy(self):
        if np is None:
            self.skipTest("numpy not available")
        ampl = self.ampl
        ampl.eval("var v_indexed{1..3};")
        ampl.var["v_indexed"][2] = np.int64(123)
        self.assertEqual(ampl.var["v_indexed"][2].value(), 123)
        ampl.eval("var v_scalar;")
        ampl.var["v_scalar"] = np.int64(456)
        self.assertEqual(ampl.var["v_scalar"].value(), 456)

    def test_constraint(self):
        load_diet_model(self.ampl)
        ampl = self.ampl
        # ampl.solve()
        self.assertEqual(
            ampl.get_constraint("diet").num_instances(), ampl.get_set("NUTR").size()
        )
        for _, con in ampl.get_constraint("diet"):
            self.assertTrue(isinstance(con.is_logical(), bool))
            con.drop()
            con.restore()
            self.assertTrue(isinstance(con.body(), float))
            self.assertTrue(isinstance(con.astatus(), str))
            self.assertEqual(con.astatus(), "in")
            con.drop()
            self.assertEqual(con.astatus(), "drop")
            con.restore()
            self.assertEqual(con.astatus(), "in")
            self.assertTrue(isinstance(con.defvar(), int))
            self.assertTrue(isinstance(con.dinit(), float))
            self.assertTrue(isinstance(con.dinit0(), float))
            self.assertTrue(isinstance(con.dual(), float))
            self.assertTrue(isinstance(con.lb(), float))
            self.assertTrue(isinstance(con.ub(), float))
            self.assertTrue(isinstance(con.lbs(), float))
            self.assertTrue(isinstance(con.ubs(), float))
            self.assertTrue(isinstance(con.ldual(), float))
            self.assertTrue(isinstance(con.udual(), float))
            self.assertTrue(isinstance(con.lslack(), float))
            self.assertTrue(isinstance(con.uslack(), float))
            self.assertTrue(isinstance(con.slack(), float))
            self.assertTrue(isinstance(con.status(), str))
            self.assertTrue(isinstance(con.sstatus(), str))
            con.setDual(0)
            self.assertEqual(con.val(), None)
        ampl.eval(
            """
            var x;
            var y;
            s.t. xy: x <==> y;
        """
        )
        self.assertTrue(ampl.get_constraint("xy").is_logical())
        self.assertTrue(isinstance(ampl.get_constraint("xy").val(), float))

    def test_set(self):
        load_diet_model(self.ampl)
        ampl = self.ampl
        self.assertEqual(ampl.get_set("FOOD").to_string(), "set FOOD;")
        self.assertEqual(str(ampl.get_set("FOOD")), "set FOOD;")
        self.assertEqual(
            ampl.get_set("NUTR").size(),
            len(ampl.get_set("NUTR").members()),
        )
        self.assertEqual(
            len(ampl.get_set("NUTR").members()),
            len(list(ampl.get_set("NUTR").members())),
        )
        self.assertTrue(ampl.get_set("NUTR").contains("A"))
        self.assertTrue(ampl.get_set("NUTR").contains(("B1",)))
        self.assertFalse(ampl.get_set("NUTR").contains("X"))
        for _, st in ampl.get_sets():
            self.assertTrue(isinstance(st.arity(), int))
            self.assertEqual(st.arity(), 1)
            self.assertTrue(isinstance(st.size(), int))
            self.assertEqual(len(st.instances()), 1)
            self.assertEqual(len(dict(st.instances())), len(list(st.instances())))
            self.assertTrue(isinstance(st.arity(), int))
        ampl.eval("model; set T{1..2}; set T2{1..2};")
        ampl.get_set("T")[1] = [-1, -2]
        self.assertEqual(ampl.get_set("T").get((1,)).get_values().to_list(), [-1, -2])
        self.assertEqual(ampl.get_set("T").find((3,)), None)
        self.assertIsInstance(ampl.get_set("T").find(1), amplpy.Set)
        ampl.get_set("T")[1].set_values([1, 2])
        self.assertEqual(ampl.get_set("T")[1].get_values().to_list(), [1, 2])
        ampl.get_set("T")[2].set_values(["1", 2])
        self.assertEqual(ampl.get_set("T")[2].get_values().to_list(), ["1", 2])
        ampl.get_set("T2")[1].set_values(ampl.get_set("T")[1].get_values())
        self.assertEqual(ampl.get_set("T2")[1].get_values().to_list(), [1, 2])
        ampl.get_set("T2")[2].set_values(ampl.get_set("T")[2].get_values())
        self.assertEqual(ampl.get_set("T2")[2].get_values().to_list(), ["1", 2])
        ampl.eval("set T3 dimen 2; set T4 dimen 2;")
        ampl.get_set("T3").set_values([(1, 2)])
        self.assertEqual(ampl.get_set("T3").get_values().to_list(), [(1, 2)])
        ampl.get_set("T3").set_values(set([("b", 1), ("a", 2)]))
        self.assertEqual(
            sorted(ampl.get_set("T3").get_values().to_list()), [("a", 2), ("b", 1)]
        )
        ampl.eval("set T5;")
        ampl.set["T5"] = set(["a", "b", "c"])
        self.assertEqual(
            sorted(ampl.get_set("T5").get_values().to_list()), ["a", "b", "c"]
        )

    def test_parameter(self):
        load_diet_model(self.ampl)
        ampl = self.ampl
        self.assertEqual(
            ampl.get_set("FOOD").size(), ampl.get_parameter("cost").num_instances()
        )
        ampl.eval(
            """
            param a;
            param b default a;
            param c symbolic;
            param d{1..2} symbolic;
            param d2{1..2} symbolic;
        """
        )
        self.assertFalse(ampl.get_parameter("a").has_default())
        self.assertTrue(ampl.get_parameter("b").has_default())
        self.assertFalse(ampl.get_parameter("a").is_symbolic())
        self.assertFalse(ampl.get_parameter("b").is_symbolic())
        self.assertTrue(ampl.get_parameter("c").is_symbolic())
        cost = ampl.get_parameter("cost")
        self.assertEqual(cost.indexarity(), 1)
        self.assertEqual(cost.get_indexing_sets(), ["FOOD"])
        self.assertEqual(cost.xref(), ["total_cost"])
        for i, food in enumerate(ampl.get_set("FOOD").members()):
            cost[food] = i + 1
            self.assertEqual(cost[food], i + 1)
            cost.set(food, i + 2)
            self.assertEqual(cost[food], i + 2)
        a = ampl.get_parameter("a")
        a.set(10)
        self.assertEqual(a.value(), 10)
        c = ampl.get_parameter("c")
        b = ampl.get_parameter("c")
        c.set("a")
        self.assertEqual(c.value(), "a")
        self.assertEqual(b.value(), "a")
        self.assertEqual(b.indexarity(), 0)
        self.assertEqual(a.is_scalar(), True)
        self.assertEqual(b.is_scalar(), True)
        self.assertEqual(c.is_scalar(), True)
        d = ampl.get_parameter("d")
        self.assertEqual(d.is_scalar(), False)
        d.set(1, "a")
        self.assertEqual(d[1], "a")
        d[1] = "aa"
        self.assertEqual(d[1], "aa")
        d.set(2, "b")
        self.assertEqual(d[2], "b")
        d.set_values({1: "x", 2: "y"})
        self.assertEqual(d[1], "x")
        self.assertEqual(d[2], "y")
        d.set_values(["xx", "yy"])
        self.assertEqual(d[1], "xx")
        self.assertEqual(d[2], "yy")
        d2 = ampl.get_parameter("d2")
        d2.set_values(d.get_values())
        self.assertEqual(d2[1], "xx")
        self.assertEqual(d2[2], "yy")
        self.assertEqual(cost.is_scalar(), False)
        ampl.eval("param cost2{FOOD};")
        self.assertTrue(isinstance(cost.get_values(), amplpy.DataFrame))
        cost2 = ampl.get_parameter("cost2")
        cost2.set_values(cost.get_values())
        for food in ampl.get_set("FOOD").members():
            self.assertEqual(cost2[food], cost[food])

    def test_parameter_numpy(self):
        if np is None:
            self.skipTest("numpy not available")
        ampl = self.ampl
        ampl.eval("param p_indexed{1..3};")
        ampl.get_parameter("p_indexed").set_values(np.array([4, 5, 6]))
        self.assertEqual(
            ampl.get_parameter("p_indexed").get_values().to_list(),
            [(1, 4), (2, 5), (3, 6)],
        )
        ampl.eval("param p_scalar;")
        ampl.get_parameter("p_scalar").set(np.int64(123))
        self.assertEqual(ampl.get_parameter("p_scalar").value(), 123)
        ampl.get_parameter("p_indexed").set(2, np.int64(456))
        self.assertEqual(ampl.get_parameter("p_indexed").get(2), 456)

    def test_parameter_numpy_array(self):
        if np is None:
            self.skipTest("numpy not available")
        ampl = self.ampl
        ampl.eval(
            r"""
        set I;
        set J;
        param p{I, J};
        """
        )
        values = np.array([1, 2, 3])
        n, k = values.shape[0], 1
        ampl.set["I"] = range(n)
        ampl.set["J"] = range(k)
        ampl.param["p"] = values
        self.assertEqual(ampl.param["p"].to_dict(), {(0, 0): 1, (1, 0): 2, (2, 0): 3})

    def test_parameter_numpy_ndarray(self):
        if np is None:
            self.skipTest("numpy not available")
        ampl = self.ampl
        ampl.eval(
            r"""
        set I;
        set J;
        param p{I, J};
        """
        )
        values = np.array(
            [
                [1, 2, 3],
                [4, 5, 6],
            ]
        )
        n, k = values.shape
        ampl.set["I"] = range(n)
        ampl.set["J"] = range(k)
        ampl.param["p"] = values
        self.assertEqual(
            ampl.param["p"].to_dict(),
            {
                (0, 0): 1,
                (0, 1): 2,
                (0, 2): 3,
                (1, 0): 4,
                (1, 1): 5,
                (1, 2): 6,
            },
        )

    def test_set_numpy(self):
        if np is None:
            self.skipTest("numpy not available")
        ampl = self.ampl
        ampl.eval("set A dimen 1;")
        ampl.get_set("A").set_values(np.array([4, 5, 6]))
        self.assertEqual(
            ampl.get_set("A").get_values().to_list(),
            [4, 5, 6],
        )
        ampl.eval("set B dimen 2;")
        ampl.get_set("B").set_values(np.array([[4, 1], [5, 2], [6, 3]]))
        self.assertEqual(
            ampl.get_set("B").get_values().to_list(),
            [(4, 1), (5, 2), (6, 3)],
        )

    def test_set_iterable(self):
        ampl = self.ampl
        ampl.eval("set A dimen 1;")
        ampl.get_set("A").set_values([1, 2, "A", "1", "2"])
        self.assertEqual(
            ampl.get_set("A").get_values().to_list(),
            [1, 2, "A", "1", "2"],
        )
        self.assertEqual(ampl.get_set("A").size(), 5)
        self.assertEqual(ampl.get_set("A").arity(), 1)
        ampl.eval("set B dimen 2;")
        ampl.get_set("B").set_values([[4, 1], [5, 2], [6, 3]])
        self.assertEqual(
            ampl.get_set("B").get_values().to_list(),
            [(4, 1), (5, 2), (6, 3)],
        )
        self.assertEqual(ampl.get_set("B").size(), 3)
        self.assertEqual(ampl.get_set("B").arity(), 2)
        with self.assertRaises(ValueError):
            ampl.get_set("A").set_values([1, (2, 1), "A", "1", "2"])
        with self.assertRaises(ValueError):
            ampl.get_set("A").set_values([(2, 1), (3, 4)])
        with self.assertRaises(ValueError):
            ampl.get_set("B").set_values([1, "A", "1", "2"])
        with self.assertRaises(ValueError):
            ampl.get_set("B").set_values([1, 2])

    def test_precision(self):
        pi = 3.1415926535897932384626433832795028841971
        ampl = self.ampl
        ampl.eval("param x;")
        ampl.get_parameter("x").set(pi)
        ampl.eval("display x;")  # invalidate cache
        self.assertEqual(ampl.param["x"].value(), pi)

    def test_infinity(self):
        inf = float("inf")
        ampl = self.ampl
        ampl.eval("param x;")
        ampl.get_parameter("x").set(inf)
        self.assertEqual(ampl.get_value("x"), inf)
        self.assertTrue(isinstance(ampl.get_value("x"), float))
        ampl.get_parameter("x").set(5)
        self.assertEqual(ampl.get_value("x"), 5)
        self.assertTrue(isinstance(ampl.get_value("x"), int))
        ampl.eval("param y{1..3};")
        ampl.get_parameter("y").set_values([inf] * 3)
        for i in range(3):
            self.assertEqual(ampl.get_value(f"y[{i+1}]"), inf)

    def test_objective(self):
        load_diet_model(self.ampl)
        ampl = self.ampl
        obj = ampl.get_objective("total_cost")
        self.assertEqual(ampl.get_current_objective().name(), "total_cost")
        self.assertEqual(len(dict(obj)), obj.num_instances())
        self.assertTrue(isinstance(obj.value(), float))
        self.assertTrue(isinstance(obj.astatus(), str))
        self.assertTrue(isinstance(obj.sstatus(), str))
        self.assertTrue(isinstance(obj.exitcode(), int))
        self.assertTrue(isinstance(obj.message(), str))
        self.assertTrue(isinstance(obj.result(), str))
        obj.drop()
        self.assertEqual(obj.astatus(), "drop")
        obj.restore()
        self.assertEqual(obj.astatus(), "in")
        self.assertTrue(obj.minimization())
        ampl.eval(
            """
            maximize A: 1;
            minimize B: 1;
            maximize C: 1;
        """
        )
        self.assertEqual(ampl.get_current_objective(), None)
        ampl.eval("objective A;")
        self.assertEqual(ampl.get_current_objective().name(), "A")
        self.assertFalse(ampl.get_current_objective().minimization())
        ampl.eval("objective B;")
        self.assertEqual(ampl.get_current_objective().name(), "B")
        self.assertTrue(ampl.get_current_objective().minimization())
        ampl.eval("objective C;")
        self.assertEqual(ampl.get_current_objective().name(), "C")
        self.assertFalse(ampl.get_current_objective().minimization())
        ampl.eval("objective A;")
        self.assertEqual(ampl.get_current_objective().name(), "A")
        self.assertFalse(ampl.get_current_objective().minimization())

    def test_set_values(self):
        ampl = self.ampl
        ampl.eval("var x{1..3};")
        d = {1: 2, 2: 5, 3: 9}
        ampl.var["x"].set_values(d)
        self.assertEqual(d, ampl.var["x"].get_values().to_dict())

        d = {1: 11, 2: 55, 3: 99}
        ampl.var["x"] = d
        self.assertEqual(d, ampl.var["x"].get_values().to_dict())

    def test_valid_data(self):
        ampl = self.ampl
        ampl.eval("set I ordered; param P{I};")
        ampl.eval("data; set I := 1 2; param P := 2 1 1 3;")
        self.assertEqual(
            ampl.get_parameter("P").get_values().to_list(), [(1, 3), (2, 1)]
        )
        self.assertEqual(ampl.get_data("P").to_list(), [(1, 3), (2, 1)])

    def test_invalid_data(self):
        ampl = self.ampl
        ampl.eval("set I ordered; param P{I};")
        ampl.eval("data; set I := 1 2; param P := 2 1 1 3 3 0;")
        with self.assertRaises(RuntimeError):
            print(ampl.get_parameter("P").get_values())
        ampl.eval("reset data; data; set I := 1 2; param P := 2 1 1 3 3 0;")
        with self.assertRaises(RuntimeError):
            print(ampl.get_data("P"))

    def test_entity_get_values_aliases(self):
        if pd is None:
            self.skipTest("pandas not available")
        ampl = self.ampl
        ampl.eval("param p{i in 1..10, j in 1..10} := i*j;")
        self.assertTrue(
            ampl.get_parameter("p")
            .to_pandas()
            .equals(ampl.get_parameter("p").get_values().to_pandas())
        )
        self.assertTrue(
            ampl.get_parameter("p")
            .to_pandas(multi_index=False)
            .equals(ampl.get_parameter("p").get_values().to_pandas(multi_index=False))
        )
        self.assertEqual(
            ampl.get_parameter("p").to_list(),
            ampl.get_parameter("p").get_values().to_list(),
        )
        self.assertEqual(
            ampl.get_parameter("p").to_list(skip_index=True),
            ampl.get_parameter("p").get_values().to_list(skip_index=True),
        )
        self.assertEqual(
            ampl.get_parameter("p").to_list(skip_index=True),
            [i * j for i in range(1, 10 + 1) for j in range(1, 10 + 1)],
        )
        self.assertEqual(
            ampl.get_parameter("p").to_dict(),
            ampl.get_parameter("p").get_values().to_dict(),
        )

    def test_stack(self):
        if pd is None:
            self.skipTest("pandas not available")
        ampl = self.ampl
        ampl.eval(
            """
        set PRODUCTS;
        set RESOURCES;
        param need {RESOURCES,PRODUCTS} >= 0;
        """
        )
        need_dict = {
            "U": {"M": 10.0, "A": 1.0, "B": 2.0},
            "V": {"M": 9.0, "A": 1.0, "B": 1.0},
        }
        PRODUCTS = ["U", "V"]
        RESOURCES = ["M", "A", "B"]
        ampl.set["PRODUCTS"] = PRODUCTS
        ampl.set["RESOURCES"] = RESOURCES
        ampl.param["need"] = pd.DataFrame(
            [
                [10, 1, 2],
                [9, 1, 1],
            ],
            columns=RESOURCES,
            index=PRODUCTS,
        ).transpose()
        self.assertEqual(
            ampl.param["need"].to_dict(),
            {
                (r, p): value
                for p, rdict in need_dict.items()
                for r, value in rdict.items()
            },
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import string
import random
import os
from amplpy import DataFrame
from . import TestBase

try:
    import numpy as np
except ImportError:
    np = None
try:
    import pandas as pd
except ImportError:
    pd = None


class TestDataFrame(TestBase.TestBase):
    """Test DataFrame."""

    def test_dataframe(self):
        # Create first dataframe (for data indexed over NUTR)
        # Add data row by row
        df1 = DataFrame("NUTR", ("n_min", "n_max"))
        df1._add_row(("A", 700, 20000))
        df1._add_row(("B1", 700, 20000))
        df1._add_row(("B2", 700, 20000))
        df1._add_row(("C", 700, 20000))
        df1._add_row(("CAL", 16000, 24000))
        df1._add_row(("NA", 0.0, 50000))

        # Create second dataframe (for data indexed over FOOD)
        # Add column by column
        df2 = DataFrame("FOOD")
        foods = ["BEEF", "CHK", "FISH", "HAM", "MCH", "MTL", "SPG", "TUR"]
        df2._set_column("FOOD", foods)
        self.assertEqual(list(df2._get_column("FOOD")), foods)
        contents = [2] * 8
        df2._add_column("f_min", contents)
        self.assertEqual(list(df2._get_column("f_min")), contents)
        contents = [10] * 8
        df2._add_column("f_max", contents)
        self.assertEqual(list(df2._get_column("f_max")), contents)
        costs = [3.19, 2.59, 2.29, 2.89, 1.89, 1.99, 1.99, 2.49]
        df2._add_column("cost", costs)
        self.assertEqual(list(df2._get_column("cost")), costs)
        labels = [random.choice(string.ascii_letters)] * 8
        df2._add_column("labels", labels)
        self.assertEqual(list(df2._get_column("labels")), labels)
        df2._add_column("empty", [])
        self.assertEqual(list(df2._get_column("empty")), [None] * 8)

        print(df2._get_column("FOOD"))
        for index in df2._get_column("FOOD"):
            print(df2._get_row(index))

        # Create third dataframe, to assign data to the AMPL entity
        # param amt{NUTR, FOOD};
        df3 = DataFrame(("NUTR", "FOOD"))
        # Populate the set columns
        nutr_with_multiplicity = [""] * 48
        food_with_multiplicity = [""] * 48
        i = 0
        for n in range(6):
            for f in range(8):
                print(df1._get_row_by_index(n)[0])
                nutr_with_multiplicity[i] = df1._get_row_by_index(n)[0]
                food_with_multiplicity[i] = foods[f]
                i += 1
        df3._set_column("NUTR", nutr_with_multiplicity)
        df3._set_column("FOOD", food_with_multiplicity)

        # Populate with all these values
        values = [
            60,
            8,
            8,
            40,
            15,
            70,
            25,
            60,
            10,
            20,
            15,
            35,
            15,
            15,
            25,
            15,
            15,
            20,
            10,
            10,
            15,
            15,
            15,
            10,
            20,
            0,
            10,
            40,
            35,
            30,
            50,
            20,
            295,
            770,
            440,
            430,
            315,
            400,
            370,
            450,
            968,
            2180,
            945,
            278,
            1182,
            896,
            1329,
            1397,
        ]
        df3._add_column("amt", values)

    def test_pandas(self):
        ampl = self.ampl
        if pd is None:
            self.skipTest("pandas not available")
        df = pd.DataFrame({"a": [1, 2], "b": [3.5, 4]}, index=["x", "y"])
        ampl.eval(
            """
            set S;
            param a{S};
            param b{S};
        """
        )
        ampl.set_data(df, "S")
        self.assertEqual(list(ampl.set["S"].members()), ["x", "y"])
        self.assertEqual(ampl.param["a"]["x"], 1)
        self.assertEqual(ampl.param["b"]["y"], 4)

        df2 = pd.DataFrame(
            {
                "a": [10, 20, 30],
            },
            index=["x", "y", "z"],
        )
        df3 = pd.DataFrame({}, index=["xx", "yy"])
        df = DataFrame.from_pandas(df)
        df2 = DataFrame.from_pandas(df2)
        df3 = DataFrame.from_pandas(df3)
        self.assertTrue(isinstance(df.to_dict(), dict))
        self.assertTrue(isinstance(df.to_list(), list))
        self.assertTrue(isinstance(df.to_pandas(), pd.DataFrame))

        self.assertEqual(df.to_list()[0][1:], (1, 3.5))
        self.assertEqual(df2.to_list()[0], ("x", 10))
        self.assertEqual(df3.to_list()[0], "xx")

        self.assertEqual(set(df.to_dict().keys()), set(["x", "y"]))
        self.assertEqual(set(df2.to_dict().keys()), set(["x", "y", "z"]))
        self.assertEqual(set(df3.to_dict().keys()), set(["xx", "yy"]))

        self.assertEqual(df.to_dict()["x"], (1, 3.5))
        self.assertEqual(df2.to_dict()["x"], 10)
        self.assertEqual(df3.to_dict()["xx"], None)

        csv_file = os.path.join(os.path.dirname(__file__), "data.csv")
        p_df = pd.read_csv(csv_file, sep=";", index_col=0)
        df = DataFrame.from_pandas(p_df)
        self.assertTrue(isinstance(df.to_dict(), dict))
        self.assertEqual(set(df.to_dict().keys()), set([1.0, 2.0, 3.0]))
        self.assertEqual(set(df.to_list()[0]), set([1.0, 0.01]))
        self.assertEqual(set(df.to_list()[1]), set([2.0, 0.02]))
        self.assertEqual(set(df.to_list()[2]), set([3.0, 0.03]))

    def test_pandas_advanced(self):
        ampl = self.ampl
        if pd is None:
            self.skipTest("pandas not available")
        ampl.eval(
            """
            set someSet := {"First", "Second"};
            param N := 5;
            param x1{someSet, 1..N} default 0;
            param x2{someSet, 1..N} default 0;
            param x3{someSet, 1..N} default 0;
            param x4{someSet, 1..N} default 0;
        """
        )

        df = pd.DataFrame(
            [[1, 2, 3, 4, 5], [6, 7, 8, 9, 0]],
            index=["First", "Second"],
            columns=[1, 2, 3, 4, 5],
        )
        ampl.param["x1"] = pd.DataFrame.from_dict(
            df.stack().to_dict(), orient="index", columns=["v"]
        )
        ampl.param["x2"] = pd.DataFrame(df.stack())
        ampl.param["x3"] = df.stack().to_dict()
        ampl.param["x4"] = df.stack()
        d1 = ampl.param["x1"].get_values().to_dict()
        d2 = ampl.param["x2"].get_values().to_dict()
        d3 = ampl.param["x3"].get_values().to_dict()
        d4 = ampl.param["x4"].get_values().to_dict()
        self.assertEqual(d1, d2)
        self.assertEqual(d2, d3)
        self.assertEqual(d3, d4)

    def test_pandas_named_columns(self):
        if pd is None:
            self.skipTest("pandas not available")
        df_unindexed = pd.DataFrame(
            [
                ["Apple", "Red", 3, 1.29],
                ["Apple", "Green", 9, 0.99],
                ["Pear", "Red", 25, 2.59],
                ["Pear", "Green", 26, 2.79],
                ["Lime", "Green", 99, 0.39],
            ],
            columns=["Fruit", "Color", "Count", "Price"],
        )
        # RangeIndex
        self.assertEqual(
            DataFrame.from_pandas(df_unindexed)._get_headers(),
            ("index0", "Fruit", "Color", "Count", "Price"),
        )

        # MultiIndex
        df_indexed = df_unindexed.set_index(["Fruit", "Color"])
        self.assertEqual(
            DataFrame.from_pandas(
                df_indexed, index_names=["Fruit", "Color"]
            )._get_headers(),
            ("Fruit", "Color", "Count", "Price"),
        )

        # Index without name
        df = pd.DataFrame(
            [[1, 2, 3, 4, 5], [6, 7, 8, 9, 0]],
            index=["First", "Second"],
            columns=[1, 2, 3, 4, 5],
        )
        self.assertEqual(
            DataFrame.from_pandas(df.stack())._get_headers(), ("index0", "index1", "0")
        )

    def test_pandas_multi_index(self):
        if pd is None:
            self.skipTest("pandas not available")
        ampl = self.ampl
        ampl.eval("param p{i in 1..100, j in 1..100} := i*j;")
        df1 = ampl.get_data("p").to_pandas()
        self.assertEqual(df1.index.nlevels, 2)
        df2 = ampl.get_data("p").to_pandas(multi_index=False)
        self.assertEqual(df2.index.nlevels, 1)
        self.assertEqual(df1.index.to_list(), df2.index.to_list())

    def test_numpy(self):
        ampl = self.ampl
        if np is None:
            self.skipTest("numpy not available")
        ampl.eval("set X;")
        arr = np.array([1, 2, 3])
        ampl.set["X"] = arr
        self.assertEqual(list(ampl.set["X"].members()), [1, 2, 3])
        ampl.eval("param p{1..3};")
        ampl.param["p"] = arr
        self.assertEqual(ampl.param["p"][2], 2)
        self.assertEqual(DataFrame.from_numpy(arr).to_list(), [1, 2, 3])
        mat = np.array([[1, 2], [3, 4], [5, 6]])
        self.assertEqual(DataFrame.from_numpy(mat).to_list(), [(1, 2), (3, 4), (5, 6)])
        self.assertEqual(DataFrame.from_numpy(mat[:, 0]).to_list(), [1, 3, 5])
        self.assertEqual(DataFrame.from_numpy(mat[:, 1]).to_list(), [2, 4, 6])

    def test_dict(self):
        dic = {"aa": "bb", "c": "a"}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {1: 2}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {1: 2, 3: 4}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {2.0: ("a", "b"), 3: ("1", "2")}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        dic = {(2.0, "c"): ("a", "b"), (3, "a"): ("1", "2")}
        self.assertEqual(dic, DataFrame.from_dict(dic).to_dict())
        df = DataFrame("x", "y")
        dic = {1: 12, 2: 23}
        df._set_values(dic)
        self.assertEqual(dic, df.to_dict())
        df = DataFrame("x", ["y", "z"])
        dic = {1: (12, 2), 2: (23, -1)}
        df._set_values(dic)
        self.assertEqual(dic, df.to_dict())
        df = DataFrame("x", ["y", "z"])
        df._set_values({1: [1, 2]})
        self.assertEqual({1: (1, 2)}, df.to_dict())

    def test_no_index(self):
        if pd is None:
            self.skipTest("pandas not available")
        df = DataFrame([], ["x", "y"])
        x = [1, 2, 3]
        y = [4, 5, 6]
        df._set_column("x", x)
        df._set_column("y", y)
        with self.assertRaises(ValueError):
            df.to_dict()
        pd_df = df.to_pandas()
        self.assertEqual(list(pd_df["x"]), x)
        self.assertEqual(list(pd_df["y"]), y)

    def test_iter(self):
        df = DataFrame("x", "y")
        dic = {1: 12, 2: 23}
        df._set_values(dic)
        for row in df:
            self.assertTrue(tuple(row) in [(1, 12), (2, 23)])

    def test_integer_values(self):
        if pd is None:
            self.skipTest("pandas not available")
        ampl = self.ampl
        df1 = ampl.get_data("{i in 1..2, j in 1..2} i*j/2")
        df1_pd = df1.to_pandas()
        df2 = ampl.get_data("{i in 1..2, j in 1..2} i*j")
        df2_pd = df2.to_pandas()

        self.assertEqual(df1_pd.index.tolist(), [(1, 1), (1, 2), (2, 1), (2, 2)])
        self.assertEqual(df1_pd["i*j/2"].tolist(), [0.5, 1.0, 1.0, 2.0])

        self.assertEqual(df2_pd.index.tolist(), [(1, 1), (1, 2), (2, 1), (2, 2)])
        self.assertEqual(df2_pd["i*j"].tolist(), [1, 2, 2, 4])

        self.assertEqual(df1.to_dict(), {(1, 1): 0.5, (1, 2): 1, (2, 1): 1, (2, 2): 2})
        self.assertEqual(df2.to_dict(), {(1, 1): 1, (1, 2): 2, (2, 1): 2, (2, 2): 4})

        self.assertEqual(df1.to_list(), [(1, 1, 0.5), (1, 2, 1), (2, 1, 1), (2, 2, 2)])
        self.assertEqual(df2.to_list(), [(1, 1, 1), (1, 2, 2), (2, 1, 2), (2, 2, 4)])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import os
from .modules import activate as activate_license
from .modules import install as install_modules
from .modules import load as load_modules
from .modules import cloud_platform_name
from .utils import (
    register_magics,
    _is_valid_uuid,
)


def _deactivate_license():
    if "AMPL_LICFILE_DEFAULT" in os.environ:
        os.environ["AMPL_LICFILE"] = os.environ["AMPL_LICFILE_DEFAULT"]
    elif "AMPL_LICFILE" in os.environ:
        del os.environ["AMPL_LICFILE"]


def _ampl_license_cell(check_callback):
    try:
        import ipywidgets as widgets
    except ImportError:
        check_callback()
        return
    from IPython.display import display

    header = widgets.Output()
    message = widgets.Output()
    version = widgets.Output()

    platform = cloud_platform_name()
    with header:
        print(
            "AMPL License UUID (you can use free https://ampl.com/ce or https://ampl.com/courses licenses):"
        )
    with message:
        ampl_lic = os.environ.get("AMPL_LICFILE", None)
        if ampl_lic is not None and platform is None:
            print(f"License license at {ampl_lic}.")
    with version:
        if check_callback:
            check_callback()

    uuid_input = widgets.Text(
        description="License UUID:", style={"description_width": "initial"}
    )

    def activate(where):
        uuid = uuid_input.value.strip()
        if where == "uuid" and len(uuid) == 0:
            return
        message.clear_output(wait=False)
        with message:
            if where == "uuid":
                if len(uuid) == 36:
                    uuid_input.value = ""  # clear the input
                    try:
                        activate_license(uuid)
                        print("License activated.")
                    except:
                        print("Failed to activate license.")
                        _deactivate_license()
                else:
                    print("Please provide a license UUID or use an existing license.")
                    return

        version.clear_output(wait=False)
        with version:
            if check_callback:
                check_callback()

    uuid_input.observe(lambda d: activate("uuid"), "value")
    display(widgets.VBox([header, widgets.HBox([uuid_input]), message, version]))


def ampl_notebook(
    modules=[],
    license_uuid=None,
    reinstall=False,
    verbose=False,
    show_license=None,
    **kwargs,
):
    try:
        from IPython import get_ipython
    except:
        get_ipython = lambda: None
    globals_ = kwargs.get("globals_", kwargs.get("g", None))
    if globals_ is None:
        if get_ipython() is not None:
            globals_ = get_ipython().user_global_ns
        else:
            globals_ = {}

    show_prompt = get_ipython() is not None and globals_ is not None
    if show_license is None:
        show_license = True

    def instantiate_ampl(print_license=True):
        from amplpy import AMPL

        ampl = AMPL()
        if print_license:
            version = ampl.option["version"]
            for row in version.split("\n"):
                if row.startswith("Licensed to "):
                    print(row)
                    break
            else:
                print(version)
        globals_["ampl"] = ampl

    if modules not in (None, []):
        install_modules(modules, reinstall=reinstall, verbose=verbose)
    else:
        modules = []

    open_modules = [
        "highs",
        "cbc",
        "couenne",
        "bonmin",
        "ipopt",
        "coin",
        "gcg",
        "scip",
        "gecode",
        "open",
        "plugins",
        "ampl",
    ]
    open_source_only = len(set(modules) - set(open_modules)) == 0

    using_default_license = license_uuid is not None and (
        license_uuid == "default" or "license-uuid" in license_uuid
    )

    # Ignore default license outside colab
    if using_default_license and cloud_platform_name() != "colab":
        using_default_license = False
        license_uuid = None

    if using_default_license:
        using_default_license = activate_license("default")
    if using_default_license:
        if show_prompt and open_source_only:
            show_prompt = False
        if not show_prompt:
            print(
                "Using default Community Edition License for Colab. "
                "Get yours at: https://ampl.com/ce"
            )

    if using_default_license:
        if show_prompt:
            _ampl_license_cell(check_callback=instantiate_ampl)
        else:
            instantiate_ampl(print_license=show_license)
    elif not _is_valid_uuid(license_uuid):
        if license_uuid not in (None, ""):
            print(
                "Please provide a valid license UUID. "
                "You can use a free https://ampl.com/ce license."
            )
            _deactivate_license()
        if show_prompt:
            _ampl_license_cell(check_callback=instantiate_ampl)
        else:
            instantiate_ampl(print_license=True)
    else:
        try:
            activate_license(license_uuid)
            instantiate_ampl(print_license=show_license)
        except:
            print("Failed to activate license.")
            _deactivate_license()
            if show_prompt:
                _ampl_license_cell(check_callback=instantiate_ampl)
            else:
                instantiate_ampl(print_license=True)

    if get_ipython() is not None:
        register_magics(ampl_object="ampl", globals_=globals_)
    return globals_.get("ampl", None)
